*
*==================================================================
*     PB-BLAS LEVEL 2 FUNCTION TEST
*==================================================================
*
      PROGRAM PBD2CHECK
*
*  -- PB-BLAS Level 2 Double precision Checker routine (version 2.0) --
*     Jaeyoung Choi, Oak Ridge National Laboratory
*     Jack Dongarra, University of Tennessee and Oak Ridge National Lab.
*     David Walker,  Oak Ridge National Laboratory
*     February 28, 1995
*
*
*     The user should modify TOTMEM to indicate the maximum amount of memory
*     in bytes his system has.  Remember to leave room in memory for operating
*     system, the BLACS buffer, etc.  INTSIZ and DBLSIZ indicate the length in
*     bytes on the given platform for an integer and a double precision real.
*     For example, on our system with 8 MB of memory, TOTMEM=6500000
*     (leaves 1.5 MB for OS, code, BLACS buffer, etc), the length of a
*     DOUBLE is 8, and an integer takes up 4 bytes.  Some playing around
*     to discover what the maximum value you can set MEMSIZ to may be required.
*     All arrays used by factorization and solve are allocated out of big
*     array called MEM.
*
      INTEGER          NTESTS, TOTMEM, MEMSIZ, ITYSIZ
      DOUBLE PRECISION ZERO, ONE
      PARAMETER        ( NTESTS = 10 )
      PARAMETER        ( TOTMEM = 2000000 )
      PARAMETER        ( ITYSIZ = 8 )
      PARAMETER        ( ZERO = 0.0D+0, ONE = 1.0D+0 )
      PARAMETER        ( MEMSIZ = TOTMEM / ITYSIZ )
*     ..
*     .. Local Scalars  ..
      LOGICAL          PBALL, PBERR
      INTEGER          ICONTXT, NOUT, M, N, MB, NB, MZ, NZ
      INTEGER          IAM, NNODES, IPA, IPC, IPX, IPY, IPZ, IPW
      INTEGER          NPROW, NPCOL, MYROW, MYCOL
      INTEGER          NMAT, NBMAT, NGRIDS, MULLEN, II, JJ, KK, INTEST
      INTEGER          IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL
      INTEGER          IASEED, IXSEED, IYSEED, INCX, INCY
      INTEGER          LCA, LCX, LCY, LRA, LRX, LRY
      INTEGER          MM, NN, IX, IY, IZ, JX, JZ
      DOUBLE PRECISION ERROR, TOL, TOLL, XMAX, TZERO
      DOUBLE PRECISION ALPHA, BETA
      CHARACTER*1      XWORK, YWORK, XCOMM, YCOMM, XYCOMM
      CHARACTER*1      DIAG, BR1ST
      CHARACTER*80     OUTFILE
*     ..
*     .. Arrays ..
      INTEGER          MVAL( NTESTS ), MBVAL( NTESTS ), MZVAL( NTESTS ),
     $                 NBVAL( NTESTS ), NPVAL( NTESTS ),
     $                 NQVAL( NTESTS ), NVAL( NTESTS ), NZVAL(NTESTS)
      DOUBLE PRECISION MEM(MEMSIZ)
*     ..
*     .. External Functions ..
      LOGICAL          LSAME
      INTEGER          IDAMAX, NUMROC
      DOUBLE PRECISION DLAMCH
      EXTERNAL         LSAME, IDAMAX, NUMROC, DLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC        MAX
*     ..
*     .. Executable Statements ..
*
*     Get starting information
*
      CALL BLACS_PINFO( IAM, NNODES )
      ICONTXT = 100
*
      CALL PBD2CHKINFO( OUTFILE, NOUT, NMAT, MVAL, NVAL, NBMAT, MBVAL,
     $                  NBVAL, MZVAL, NZVAL, NGRIDS, NPVAL, NQVAL,
     $                  ALPHA, BETA, XCOMM, YCOMM, XWORK, YWORK, DIAG,
     $                  BR1ST, MULLEN, IAROW, IACOL, IXROW, IXCOL,
     $                  IYROW, IYCOL, INCX, INCY, IASEED, IXSEED,
     $                  IYSEED, MEM, IAM, NNODES )
*
*     Test only for INCX > 0 & INCY > 0.
*
      IF(      INCX.LT.0 ) THEN
        IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'INCX'
        GO TO 40
      ELSE IF( INCY.LT.0 ) THEN
        IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'INCY'
        GO TO 40
      END IF
*
*     Generate dirty data (not used)
*
      CALL PDMATGEN( ICONTXT, 'No', 'No', 300, 300, 10, 10, MEM(1), 300,
     $               0, 0, IASEED, 0, 300, 0, 300, 0, 0, 1, 1 )
*
      INTEST = 0
      PBALL = .FALSE.
      PBERR = .FALSE.
      TZERO = ZERO
*
*     Loop over different processor grids
*
      DO 30 II = 1, NGRIDS
*
        NPROW = NPVAL(II)
        NPCOL = NQVAL(II)
*
        IF( NPROW.LT.1                        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW'
          GO TO 30
        ELSE IF( NPCOL.LT.1                   ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPCOL'
          GO TO 30
        ELSE IF( NPROW*NPCOL.GT.NNODES        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW*NPCOL'
          GO TO 30
        ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IAROW'
          GO TO 30
        ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IACOL'
          GO TO 30
        ELSE IF( IXROW.LT.0 .OR. IXROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IXROW'
          GO TO 30
        ELSE IF( IXCOL.LT.0 .OR. IXCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IXCOL'
          GO TO 30
        ELSE IF( IYROW.LT.0 .OR. IYROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IYROW'
          GO TO 30
        ELSE IF( IYCOL.LT.0 .OR. IYCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IYCOL'
          GO TO 30
        END IF
*
*       Define processor grid
*
        CALL BLACS_GET( -1, 0, ICONTXT )
        CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', NPROW, NPCOL )
        CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*       Go to bottom of processor grid loop if this case doesn't use my node
*
        IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 30
*
        DO 20 JJ = 1, NMAT
*
          M = MVAL(JJ)
          N = NVAL(JJ)
*
          IF( M.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'M'
            GO TO 20
          ELSE IF( N.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'N'
            GO TO 20
          END IF
*
          DO 10 KK = 1, NBMAT
*
            MB = MBVAL(KK)
            NB = NBVAL(KK)
            MZ = MZVAL(KK)
            NZ = NZVAL(KK)
*
            IF( MB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MB'
              GO TO 10
            ELSE IF( NB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NB'
              GO TO 10
            ELSE IF( MZ.LT.0 .OR. MZ.GE.MB ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MZ'
              GO TO 10
            ELSE IF( NZ.LT.0 .OR. NZ.GE.NB ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NZ'
              GO TO 10
            END IF
*
            IF( IAM.EQ.0 ) THEN
              WRITE(NOUT,2000)
              WRITE(NOUT,3000) NPROW, NPCOL, M, N, MB, NB, MZ, NZ
            END IF
*
*==================================================================
*           PBDTRNV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            NN  = N + NZ
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBDTRNV ( CASE 1 )
*
            IZ = 0
            IF( MYROW.EQ.IXROW ) IZ = NZ * INCX
            JZ = 0
            IF( MYCOL.EQ.IYCOL ) JZ = NZ * INCY
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, ZERO, MEM(IPY+JZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPX+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 1, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 2 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 2, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 3 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, 0, IYCOL, IYSEED, 0, 1, 0, LCY, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PBDTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1, -1,
     $                    IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 3, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 4 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 4, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 5 )
*
            JZ = 0
            IF( MYCOL.EQ.IXCOL ) JZ = NZ * INCX
            IZ = 0
            IF( MYROW.EQ.IYROW ) IZ = NZ * INCY
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, ZERO, MEM(IPY+IZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPX+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 5, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 6 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 6, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 7 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, 0, IYSEED, 0, LRY, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 7, IAM, PBERR )
*
*           Test PBDTRNV ( CASE 8 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1,
     $                     IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBDVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRNV ', 8, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDTRNV ', 8, IAM, PBERR )
*
*==================================================================
*           PBDGEMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
            MM  = M + MZ
            NN  = N + NZ
            LRA = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = MZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ * LRA
*
*  ======   Results for the sequential computation (Test 1 - Test 4)
*
            IPZ = 1
            IPX = MM + IPZ
            IPA = NN + IPX
            IPW = MM * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL DGEMV( 'No', M, N, ALPHA, MEM(IPA+MM*NZ+MZ), MAX(1,MM),
     $                  MEM(IPX+NZ), 1, BETA, MEM(IPZ+MZ), 1 )
*
*           Test PBDGEMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IXROW ) IX = NZ * INCX
            IY = 0
            IF( MYROW.EQ.IAROW ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGEMV( ICONTXT, 'No', 'Col', 'Col', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IAROW, IYCOL,
     $                    ' ', ' ', YWORK, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 1, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 2 )
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( MM, MB, MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IXROW ) IX = NZ * INCX
            IY = 0
            IF( MYCOL.EQ.IYCOL ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGEMV( ICONTXT, 'No', 'Col', 'Row', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL,
     $                    ' ', ' ', ' ', MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 2, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IACOL ) IX = NZ * INCX
            IY = 0
            IF( MYROW.EQ.IAROW ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGEMV( ICONTXT, 'No', 'Row', 'Col', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IACOL, IAROW, IYCOL,
     $                    XCOMM, XWORK, YWORK, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 3, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( MM, MB, MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IACOL ) IX = NZ * INCX
            IY = 0
            IF( MYCOL.EQ.IYCOL ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGEMV( ICONTXT, 'No', 'Row', 'Row', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IACOL, IYROW, IYCOL,
     $                    ' ', XWORK, ' ', MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 4, IAM, PBERR )
*
*  ======   Results for the sequential computation (Test 5 - Test 8)
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = MM + IPX
            IPW = MM * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DGEMV( 'Trans', M, N, ALPHA, MEM(IPA+MM*NZ+MZ),
     $                  MAX(1,MM), MEM(IPX+MZ), 1, BETA, MEM(IPZ+NZ),
     $                  1 )
*
*           Test PBDGEMV ( CASE 5 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGEMV( ICONTXT, 'Trans', 'Col', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IYCOL,
     $                    ' ', XWORK, ' ', MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 5, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 6 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGEMV( ICONTXT, 'Trans', 'Col', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IACOL,
     $                    XCOMM, XWORK, YWORK, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 6, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 7 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGEMV( ICONTXT, 'Trans', 'Row', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL,
     $                    ' ', ' ', ' ', MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 7, IAM, PBERR )
*
*           Test PBDGEMV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGEMV( ICONTXT, 'Trans', 'Row', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IACOL,
     $                    ' ', ' ', YWORK, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGEMV ', 8, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDGEMV ', 8, IAM, PBERR )
*
*==================================================================
*           PBDGER TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            MM  = M + MZ
            NN  = N + NZ
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
*
            LRA = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = MZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ * LRA
*
*  ======   Results for the sequential computation
*
            IPC = 1
            IPX = MM * NN + IPC
            IPY = MM + IPX
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPC), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DGER( M, N, ALPHA, MEM(IPX+MZ), 1, MEM(IPY+NZ), 1,
     $                 MEM(IPC+MM*NZ+MZ), MAX(1,MM) )
*
*           Test PBDGER ( CASE 1 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1)+IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGER( ICONTXT, 'Col', 'Col', M, N, MB, NB, MZ, NZ,
     $                   ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                   MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                   IYROW, IYCOL, IAROW, IACOL, BR1ST, XCOMM,
     $                   XWORK, ' ', ' ', MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGER  ', 1, IAM, PBERR )
*
*           Test PBDGER ( CASE 2 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGER( ICONTXT, 'Col', 'Row', M, N, MB, NB, MZ, NZ,
     $                   ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                   MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                   IYROW, IACOL, IAROW, IACOL, BR1ST, XCOMM,
     $                   XWORK, YCOMM, YWORK, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGER  ', 2, IAM, PBERR )
*
*           Test PBDGER ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDGER( ICONTXT, 'Row', 'Col', M, N, MB, NB, MZ, NZ,
     $                   ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                   MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                   IYROW, IYCOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                   ' ', ' ', MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGER  ', 3, IAM, PBERR )
*
*           Test PBDGER ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDGER( ICONTXT, 'Row', 'Row', M, N, MB, NB, MZ, NZ,
     $                   ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                   MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                   IYROW, IACOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                   YCOMM, YWORK, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDGER  ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDGER  ', 4, IAM, PBERR )
*
*==================================================================
*           PBDSYMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = NN + IPX
            IPW = NN * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DSYMV( 'Upper', N, ALPHA, MEM(IPA+NN*NZ+NZ), MAX(1,NN),
     $                  MEM(IPX+NZ), 1, BETA, MEM(IPZ+NZ), 1 )
*
*           Test PBDSYMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDSYMV( ICONTXT, 'Upper', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, BETA,
     $                    MEM(IPY+IZ*INCY), INCY, IAROW, IACOL, IXCOL,
     $                    IYCOL, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ*INCY),
     $                    INCY,  MEM(IPZ+NZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYMV ', 1, IAM, PBERR )
*
*           Test PBDSYMV ( CASE 2 )
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDSYMV( ICONTXT, 'Lower', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, BETA,
     $                    MEM(IPY+IZ*INCY), INCY, IAROW, IACOL, IXCOL,
     $                    IYCOL, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYMV ', 2, IAM, PBERR )
*
*           Test PBDSYMV ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYMV( ICONTXT, 'Upper', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, BETA,
     $                    MEM(IPY+JZ*INCY), INCY, IAROW, IACOL, IXROW,
     $                    IYROW, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYMV ', 3, IAM, PBERR )
*
*           Test PBDSYMV ( CASE 4 )
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYMV( ICONTXT, 'Lower', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, BETA,
     $                    MEM(IPY+JZ*INCY), INCY, IAROW, IACOL, IXROW,
     $                    IYROW, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYMV ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDSYMV ', 4, IAM, PBERR )
*
*==================================================================
*           PBDSYR TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (Test 1 - Test 2)
*
            IPC = 1
            IPX = NN * NN + IPC
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DSYR( 'Upper', N, ALPHA, MEM(IPX+NZ), 1,
     $                 MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBDSYR ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDSYR( ICONTXT, 'Upper', 'Col', N, NB, NZ, ALPHA,
     $                   MEM(IPX+IZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXCOL, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR  ', 1, IAM, PBERR )
*
*           Test PBDSYR ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYR( ICONTXT, 'Upper', 'Row', N, NB, NZ, ALPHA,
     $                   MEM(IPX+JZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXROW, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR  ', 2, IAM, PBERR )
*
*           Results for the sequential computation (Test 3 - Test 4)
*
            IPC = 1
            IPX = NN * NN + IPC
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DSYR( 'Lower', N, ALPHA, MEM(IPX+NZ), 1,
     $                 MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBDSYR ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDSYR( ICONTXT, 'Lower', 'Col', N, NB, NZ, ALPHA,
     $                   MEM(IPX+IZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXCOL, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR  ', 3, IAM, PBERR )
*
*           Test PBDSYR ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYR( ICONTXT, 'Lower', 'Row', N, NB, NZ, ALPHA,
     $                   MEM(IPX+JZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXROW, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR  ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDSYR  ', 4, IAM, PBERR )
*
*==================================================================
*           PBDSYR2 TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            XYCOMM = XCOMM
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (Test 1 - Test 2)
*
            IPC = 1
            IPX = NN * NN + IPC
            IPY = NN + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DSYR2( 'Upper', N, ALPHA, MEM(IPX+NZ), 1, MEM(IPY+NZ),
     $                  1, MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBDSYR2 ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LRY-1)*INCY+1) + IPY
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDSYR2( ICONTXT, 'Upper', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPX+IZ*INCX), INCX, MEM(IPY+IZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXCOL,
     $                    IYCOL, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR2 ', 1, IAM, PBERR )
*
*           Test PBDSYR2 ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LCY-1)*INCY+1) + IPY
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYR2( ICONTXT, 'Upper', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPX+JZ*INCX), INCX, MEM(IPY+JZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXROW,
     $                    IYROW, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR2 ', 2, IAM, PBERR )
*
*           Results for the sequential computation (Test 3 - Test 4)
*
            IPC = 1
            IPX = NN * NN + IPC
            IPY = NN + IPX
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DSYR2( 'Lower', N, ALPHA, MEM(IPX+NZ), 1, MEM(IPY+NZ),
     $                  1, MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBDSYR2 ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LRY-1)*INCY+1) + IPY
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL DVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBDSYR2( ICONTXT, 'Lower', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPX+IZ*INCX), INCX, MEM(IPY+IZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXCOL,
     $                    IYCOL, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR2 ', 3, IAM, PBERR )
*
*           Test PBDSYR2 ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LCY-1)*INCY+1) + IPY
*
            CALL PDMATGEN( ICONTXT, 'Symm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDSYR2( ICONTXT, 'Lower', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPX+JZ*INCX), INCX, MEM(IPY+JZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXROW,
     $                    IYROW, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PDMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDSYR2 ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDSYR2 ', 4, IAM, PBERR )
*
*==================================================================
*           PBDTRMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (TEST 1 - TEST 2)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DTRMV( 'Upper', 'No', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBDTRMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 1, IAM, PBERR )
*
*           Test PBDTRMV ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 2, IAM, PBERR )
*
*           Results for the sequential computation (TEST 3 - TEST 4)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DTRMV( 'Lower', 'No', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBDTRMV ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 3, IAM, PBERR )
*
*           Test PBDTRMV ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA,  MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 4, IAM, PBERR )
*
*           Results for the sequential computation (TEST 5 - TEST 6)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DTRMV( 'Upper', 'Trans', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBDTRMV ( CASE 5 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 5, IAM, PBERR )
*
*           Test PBDTRMV ( CASE 6 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 6, IAM, PBERR )
*
*           Results for the sequential computation (TEST 7 - TEST 8)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL DTRMV( 'Lower', 'Trans', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBDTRMV ( CASE 7 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 7, IAM, PBERR )
*
*           Test PBDTRMV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBDTRMV ', 8, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDTRMV ', 8, IAM, PBERR )
*
*==================================================================
*           PBDTRSV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Test PBDTRSV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRSV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 1, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 2 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRSV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 2, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRSV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 3, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 4 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL DVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBDTRSV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 4, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 5 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRSV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 5, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 6 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRSV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1,  0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 6, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 7 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRSV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 7, IAM, PBERR )
*
*           Test PBDTRSV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PDMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBDTRSV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IDAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBDTRMV
*
            CALL PBDTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PDMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PDVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBDTRSV ', 8, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBDTRSV ', 8, IAM, PBERR )
*
            INTEST = INTEST + 1
            IF( IAM.EQ.0 ) WRITE( NOUT, 4000 ) INTEST
   10     CONTINUE
   20   CONTINUE
        CALL BLACS_GRIDEXIT( ICONTXT )
   30 CONTINUE
   40 CONTINUE
*
      IF( IAM.EQ.0 ) THEN
         IF( PBALL ) THEN
            WRITE( NOUT, 5000 )
         ELSE
            WRITE( NOUT, 6000 ) INTEST
         END IF
         WRITE( NOUT, 7000 )
      END IF
*
      CALL BLACS_EXIT( 0 )
*
 1000 FORMAT( /, 'Illegal input value of ', A11 )
 2000 FORMAT(/, '    P    Q     M    N    MB  NB  MZ  NZ')
 3000 FORMAT(1X,I4,1X,I4,3X,I4,1X,I4,2X,I3,1X,I3,1X,I3,1X,I3)
 4000 FORMAT( '-- PBD2CHECK TEST SET (', I4, ' ) IS COMPLETED --')
 5000 FORMAT( /, '== THERE WERE FAILURES IN THIS RUN ==')
 6000 FORMAT( /, '== THERE WERE NO FAILURES IN (', I4, ' ) ',
     $        'TEST SETS ==')
 7000 FORMAT( /, 'End of PBD2CHECK TESTS.' )
*
      STOP
*
*     End of PBD2CHECK
*
      END
*
*==================================================================
*     SUBROUTINE DVREDIST  -- change INCX1 to INCX2
*==================================================================
*
      SUBROUTINE DVREDIST( N, BETA, X, INCX1, INCX2 )
*
*  -- PB-BLAS Auxiliary Routine (version 2.0) --
*     Oak Ridge National Laboratory and University of Tennessee.
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      INTEGER              N, INCX1, INCX2
      DOUBLE PRECISION     BETA
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION     X( * )
*     ..
*     .. Exteranl Subroutine ..
      EXTERNAL             DSCAL
*     ..
*     .. Parameters ..
      DOUBLE PRECISION     ZERO
      PARAMETER            ( ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER              I, IX1, IX2
*     ..
*
      IF( BETA.NE.ZERO ) THEN
        IF( INCX1.LT.INCX2 ) THEN
          IX1 = (N-1) * INCX1 + 1
          IX2 = (N-1) * INCX2 + 1
*
          DO 10 I = 1, N
            X( IX2 ) = BETA * X( IX1 )
            IX1 = IX1 - INCX1
            IX2 = IX2 - INCX2
   10     CONTINUE
*
        ELSE IF( INCX1.GT.INCX2 ) THEN
          IX1 = 1
          IX2 = 1
*
          DO 20 I = 1, N
            X( IX2 ) = BETA * X( IX1 )
            IX1 = IX1 + INCX1
            IX2 = IX2 + INCX2
   20     CONTINUE
*
        ELSE
          CALL DSCAL( N, BETA, X, INCX1 )
        END IF
*
      ELSE
        IX2 = 1
        DO 30 I = 1, N
            X( IX2 ) = ZERO
            IX2 = IX2 + INCX2
   30   CONTINUE
      END IF
*
      RETURN
*
*     End of DVREDIST
*
      END
