      PROGRAM PBS3CHECK
*
*  -- PB-BLAS Level 3 Real Single precision Checker routine (version 2.0) --
*     Jaeyoung Choi, Oak Ridge National Laboratory
*     Jack Dongarra, University of Tennessee and Oak Ridge National Lab.
*     David Walker,  Oak Ridge National Laboratory
*     February 28, 1995
*
*
*     The user should modify TOTMEM to indicate the maximum amount of memory
*     in bytes his system has.  Remember to leave room in memory for operating
*     system, the BLACS buffer, etc.  INTSIZ and DBLSIZ indicate the length in
*     bytes on the given platform for an integer and a double precision real.
*     For example, on our system with 8 MB of memory, TOTMEM=6500000
*     (leaves 1.5 MB for OS, code, BLACS buffer, etc), the length of a
*     DOUBLE is 8, and an integer takes up 4 bytes.  Some playing around
*     to discover what the maximum value you can set MEMSIZ to may be required.
*     All arrays used by factorization and solve are allocated out of big
*     array called MEM.
*
*     ..
*     .. Parameters ..
      INTEGER          NTESTS, TOTMEM, MEMSIZ, ITYSIZ
      REAL             ZERO, ONE
      PARAMETER        ( NTESTS = 10 )
      PARAMETER        ( TOTMEM = 2000000 )
      PARAMETER        ( ITYSIZ = 4 )
      PARAMETER        ( ZERO = 0.0E+0, ONE = 1.0E+0 )
      PARAMETER        ( MEMSIZ = TOTMEM / ITYSIZ )
*     ..
*     .. Local Scalars  ..
      LOGICAL          PBALL, PBERR
      INTEGER          ICONTXT, NOUT, M, N, K, MB, NB, KB
      INTEGER          IAM, NNODES, IPA, IPB, IPC, IPD, IPW
      INTEGER          NPROW, NPCOL, MYROW, MYCOL, II, IX, JJ, JX, KK
      INTEGER          NMAT, NBMAT, NGRIDS, MULLEN, INTEST
      INTEGER          IAROW, IACOL, IBROW, IBCOL, ICROW, ICCOL
      INTEGER          IASEED, IBSEED, ICSEED
      INTEGER          LCA, LCB, LCC, LRA, LRB, LRC
      INTEGER          IAPOS, IBPOS, ICPOS, IMROW, IMCOL
      REAL             ERROR, TOL, TOLL, TZERO, XMAX
      REAL             ALPHA, BETA
      CHARACTER*1      AWORK, BWORK, CWORK, ACOMM, BCOMM, CCOMM, ABCOMM
      CHARACTER*1      DIAG, BR1ST
      CHARACTER*80     OUTFILE
*     ..
*     .. Arrays ..
      INTEGER          KBVAL( NTESTS ), KVAL( NTESTS ), MVAL( NTESTS ),
     $                 MBVAL( NTESTS ), NBVAL( NTESTS ),
     $                 NPVAL( NTESTS), NQVAL( NTESTS ), NVAL( NTESTS )
      REAL             MEM( MEMSIZ )
*     ..
*     .. External Functions ..
      LOGICAL          LSAME
      INTEGER          ISAMAX, NUMROC
      REAL             SLAMCH
      EXTERNAL         LSAME, ISAMAX, NUMROC, SLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC        ABS, MAX
*     ..
*     .. Executable Statements ..
*
*     Get starting information
*
      CALL BLACS_PINFO( IAM, NNODES )
*
      CALL PBS3CHKINFO( OUTFILE, NOUT, NMAT, MVAL, NVAL, KVAL, NBMAT,
     $                  MBVAL, NBVAL, KBVAL, NGRIDS, NPVAL, NQVAL,
     $                  ALPHA, BETA, ACOMM, BCOMM, AWORK, BWORK, CWORK,
     $                  DIAG, BR1ST, MULLEN, IAROW, IACOL, IBROW, IBCOL,
     $                  ICROW, ICCOL, IASEED, IBSEED, ICSEED, MEM, IAM,
     $                  NNODES )
*
*     Generate dirty data (not used)
*
      CALL PSMATGEN( ICONTXT, 'No', 'No', 300, 300, 10, 10, MEM(1), 300,
     $               0, 0, IASEED, 0, 300, 0, 300, 0, 0, 1, 1 )
*
      INTEST = 0
      PBALL = .FALSE.
      PBERR = .FALSE.
      TZERO = ZERO
*
*     Loop over different processor grids
*
      DO 30 II = 1, NGRIDS
*
        NPROW = NPVAL(II)
        NPCOL = NQVAL(II)
*
        IF( NPROW.LT.1                        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW'
          GO TO 30
        ELSE IF( NPCOL.LT.1                   ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPCOL'
          GO TO 30
        ELSE IF( NPROW*NPCOL.GT.NNODES        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW*NPCOL'
          GO TO 30
        ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IAROW'
          GO TO 30
        ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IACOL'
          GO TO 30
        ELSE IF( IBROW.LT.0 .OR. IBROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IBROW'
          GO TO 30
        ELSE IF( IBCOL.LT.0 .OR. IBCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IBCOL'
          GO TO 30
        ELSE IF( ICROW.LT.0 .OR. ICROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'ICROW'
          GO TO 30
        ELSE IF( ICCOL.LT.0 .OR. ICCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'ICCOL'
          GO TO 30
        END IF
*
*       Define processor grid
*
        CALL BLACS_GET( -1, 0, ICONTXT )
        CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', NPROW, NPCOL )
        CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*       Go to bottom of processor grid loop if this case doesn't use my node
*
        IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 30
*
        DO 20 JJ = 1, NMAT
*
          M = MVAL(JJ)
          N = NVAL(JJ)
          K = KVAL(JJ)
*
          IF( M.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'M'
            GO TO 20
          ELSE IF( N.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'N'
            GO TO 20
          ELSE IF( K.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'K'
            GO TO 20
          END IF
*
          DO 10 KK = 1, NBMAT
            MB = MBVAL(KK)
            NB = NBVAL(KK)
            KB = KBVAL(KK)
*
            IF( MB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MB'
              GO TO 10
            ELSE IF( NB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NB'
              GO TO 10
            ELSE IF( NB.LT.0 .OR. KB.LT.K ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'KB'
              GO TO 10
            END IF
*
            IF( IAM.EQ.0 ) THEN
              WRITE(NOUT,2000)
              WRITE(NOUT,3000) NPROW, NPCOL, M, N, K, MB, NB, KB
            END IF
*
*==================================================================
*           PBSTRAN TEST
*==================================================================
*
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
*
*           Test PBSTRAN ( CASE 1 )
*
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), ZERO, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPA),
     $                     MAX(1,K), ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 1, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBSMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 2, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 3 )
*
            IPC = 1
            IPA = K * LCC + IPC
            IPW = K * LRA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, 0, ICCOL, ICSEED, 0, K, 0, LCC,
     $                     0, MYCOL, 1, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, -1, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), MAX(1,K), 0, 0, IASEED, 0, K, 0, N,
     $                     0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     MAX(1,K), 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1,
     $                     1 )
            CALL PBSMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 3, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LRA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBSMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 4, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 5 )
*
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), ZERO, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPA), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 5, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 6 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBSMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 6, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 7 )
*
            IPC = 1
            IPA = K * LRC + IPC
            IPW = K * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, 0, ICSEED, 0, LRC, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PBSTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBSMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                     BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 7, IAM, PBERR )
*
*           Test PBSTRAN ( CASE 8 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N,  0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBSMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRAN ', 8, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBSTRAN ', 8, IAM, PBERR )
*
*==================================================================
*           PBSGEMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0E0 * REAL(MAX(M,N)) * SLAMCH('eps')
*
*           Test PBSGEMM ( CASE 1 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'C', 'No', 'No', M, N, K, MB, NB, KB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IACOL, IBROW, IMCOL, IMROW, IMCOL, BR1ST,
     $                    ACOMM, AWORK, BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'No', M, N, K, ALPHA, MEM(IPA), MAX(1,M),
     $                  MEM(IPB), MAX(1,K), BETA, MEM(IPD), MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 1, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 2 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = LRC * K + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, IMROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'A', 'No', 'No', M, K, N, MB, KB, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IMCOL, IBROW, IBCOL, IMROW, ICCOL, ' ', ' ',
     $                    ' ', ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'No', M, K, N, ALPHA, MEM(IPA), MAX(1,M),
     $                  MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 2, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 3 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'B', 'No', 'No', K, N, M, KB, NB, MB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IMROW, IMCOL, ICROW, IMCOL, ' ', ' ',
     $                    ' ', ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     M, 0, 0, IBSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'No', K, N, M, ALPHA, MEM(IPA), MAX(1,K),
     $                  MEM(IPB), MAX(1,M), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 3, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 4 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'C', 'Trans', 'No', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBROW, IMCOL, IMROW, IMCOL, BR1ST, ' ',
     $                    ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'No', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 4, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 5 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRB * K + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IMROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'A', 'Trans', 'No', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IMROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'No', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 5, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 6 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPB
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'B', 'Trans', 'No', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IMROW,
     $                    IACOL, IMROW, IMCOL, ICROW, IMCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     M, 0, 0, IBSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'No', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,M), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 6, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 7 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'C', 'No', 'Trans', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ACOMM, AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'Trans', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 7, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 8 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = LRC * K + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, IMROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'A', 'No', 'Trans', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IMCOL, IBROW, IMCOL, IMROW, ICCOL, ' ', ' ',
     $                    ' ', BCOMM, BWORK, CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'Trans', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 8, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 9 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'B', 'No', 'Trans', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IMCOL, IMROW, IMCOL, ICROW, ICCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'No', 'Trans', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 9, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 10 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'C', 'Trans', 'Trans', M, N, K, MB,
     $                    NB, KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'Trans', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 10, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 11 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'A', 'Trans', 'Trans', M, K, N, MB,
     $                    KB, NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IBROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'Trans', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PSMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 11, IAM, PBERR )
*
*           Test PBSGEMM ( CASE 12 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSGEMM( ICONTXT, 'B', 'Trans', 'Trans', K, N, M, KB,
     $                    NB, MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IMROW, IMCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PSMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SGEMM( 'Trans', 'Trans', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSGEMM ', 12, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBSGEMM ', 12, IAM, PBERR )
*
*==================================================================
*           PBSSYMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
*
*           Test PBSSYMM ( CASE 1 )
*
            IBPOS = IBCOL
            ICPOS = ICCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( K, KB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Matrix', 'Left', 'Lower', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL SSYMM( 'Left', 'Lower', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 1, IAM, PBERR )
*
*           Test PBSSYMM ( CASE 2 )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Matrix', 'Left', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL SSYMM( 'Left', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 2, IAM, PBERR )
*
*           Test PBSSYMM ( CASE 3 )
*
            IBPOS = IBROW
            ICPOS = ICROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Matrix', 'Right', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SSYMM( 'Right', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 3, IAM, PBERR )
*
*           Test PBSSYMM ( CASE 4 )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Matrix', 'Right', 'Upper', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SSYMM( 'Right', 'Upper', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 4, IAM, PBERR )
*
*           Test PBSSYMM ( CASE 5 )
*
            IBPOS = IBCOL
            ICPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( N, NB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Block', 'Left', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB), MAX(1,K),
     $                    BETA, MEM(IPC), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ICPOS, ACOMM, AWORK, ' ', 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL SSYMM( 'Left', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 5, IAM, PBERR )
*
*           Test PBSSYMM ( CASE 6 )
*
            IBPOS = IBROW
            ICPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYMM( ICONTXT, 'Block', 'Right', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBPOS, ICPOS, ACOMM, AWORK, ' ', 0,
     $                    MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL SSYMM( 'Right', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYMM ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBSSYMM ', 6, IAM, PBERR )
*
*==================================================================
*           PBSSYRK TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
*
*           Test PBSSYRK ( CASE 1 )
*
            IAPOS = IACOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRC), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYRK( 'Lower', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                  BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 1, IAM, PBERR )
*
*           Test PBSSYRK ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRC), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM,
     $                    AWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYRK( 'Upper', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                  BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 2, IAM, PBERR )
*
*           Test PBSSYRK ( CASE 3 )
*
            IAPOS = IAROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Matrix', 'Lower', 'Trans', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYRK( 'Lower', 'Trans', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 3, IAM, PBERR )
*
*           Test PBSSYRK ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Matrix', 'Upper', 'Trans', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYRK( 'Upper', 'Trans', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 4, IAM, PBERR )
*
*           Test PBSSYRK ( CASE 5 )
*
            IAPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRC * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL SSYRK( 'Upper', 'No', K, N, ALPHA, MEM(IPA), MAX(1,K),
     $                  BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 5, IAM, PBERR )
*
*           Test PBSSYRK ( CASE 6 )
*
            IAPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRA * LCC + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYRK( ICONTXT, 'Block', 'Lower', 'Trans', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), BETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL SSYRK( 'Lower',  'Trans', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYRK ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBSSYRK ', 6, IAM, PBERR )
*
*==================================================================
*           PBSSYR2K TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
            ABCOMM = ACOMM
*
*           Test PBSSYR2K ( CASE 1 )
*
            IAPOS  = IACOL
            IBPOS  = IBCOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), BETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYR2K( 'Lower', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 1, IAM, PBERR )
*
*           Test PBSSYR2K ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), BETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYR2K( 'Upper', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 2, IAM, PBERR )
*
*           Test PBSSYR2K ( CASE 3 )
*
            IAPOS = IAROW
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Matrix', 'Lower', 'Trans', N,
     $                     K, NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYR2K( 'Lower', 'Trans', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 3, IAM, PBERR )
*
*           Test PBSSYR2K ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Matrix', 'Upper', 'Trans', N, K,
     $                     NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL SSYR2K( 'Upper', 'Trans', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 4, IAM, PBERR )
*
*           Test PBSSYR2K ( CASE 5 )
*
            IAPOS = IACOL
            IBPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRC * LCA + IPA
            IPW = LRC * LCB + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL SSYR2K( 'Upper', 'No', K, N, ALPHA, MEM(IPA), MAX(1,K),
     $                   MEM(IPB), MAX(1,K), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 5, IAM, PBERR )
*
*           Test PBSSYR2K ( CASE 6 )
*
            IAPOS = IAROW
            IBPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRA * LCC + IPA
            IPW = LRB * LCC + IPB
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSSYR2K( ICONTXT, 'Block', 'Lower', 'Trans', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                     MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL SSYR2K( 'Lower',  'Trans', K, N, ALPHA, MEM(IPA),
     $                   MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                   MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSSYR2K ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBSSYR2K ', 6, IAM, PBERR )
*
*==================================================================
*           PBSTRMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
*
*           Test PBSTRMM ( CASE 1 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRMM( 'Left', 'Lower', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 1, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 2 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRMM( 'Left', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 2, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 3 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRMM( 'Left', 'Lower', 'Trans', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 3, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 4 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRMM( 'Left', 'Upper', 'Trans', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 4, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 5 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRMM( 'Right', 'Lower', 'No', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 5, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 6 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRMM( 'Right', 'Upper', 'No', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 6, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 7 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRMM( 'Right', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 7, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 8 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRMM( 'Right', 'Upper', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 8, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 9 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Block', 'Left', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRMM( 'Left', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 9, IAM, PBERR )
*
*           Test PBSTRMM ( CASE 10 )
*
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRMM( ICONTXT, 'Block', 'Right', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,LRB), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRMM( 'Right', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRB,
     $                     MEM(IPD), N, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRMM ', 10, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBSTRMM ', 10, IAM, PBERR )
*
*==================================================================
*           PBSTRSM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0E0 * REAL(N) * SLAMCH('eps')
*
*           Test PBSTRSM ( CASE 1 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 1, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 2 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 2, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 3 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 3, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 4 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 4, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 5 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 5, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 6 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 6, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 7 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 7, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 8 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+ISAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBSTRMM
*
            CALL PBSTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 8, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 9 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Block', 'Left', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL STRSM( 'Left', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PSMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 9, IAM, PBERR )
*
*           Test PBSTRSM ( CASE 10 )
*
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBSTRSM( ICONTXT, 'Block', 'Right', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,LRB), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PSMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PSMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL STRSM( 'Right', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,N) )
*
            CALL PSMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRB,
     $                     MEM(IPD), N, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBSTRSM ', 10, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBSTRSM ', 10, IAM, PBERR )
*
            INTEST = INTEST + 1
            IF( IAM.EQ.0 ) WRITE( NOUT, 4000 ) INTEST
   10     CONTINUE
   20   CONTINUE
        CALL BLACS_GRIDEXIT( ICONTXT )
   30 CONTINUE
*
      IF( IAM.EQ.0 ) THEN
         IF( PBALL ) THEN
            WRITE( NOUT, 5000 )
         ELSE
            WRITE( NOUT, 6000 ) INTEST
         END IF
         WRITE( NOUT, 7000 )
      END IF
*
      CALL BLACS_EXIT( 0 )
*
 1000 FORMAT( /, 'Illegal input value of ', A11 )
 2000 FORMAT(/, '    P    Q     M    N    K    MB  NB  KB')
 3000 FORMAT(1X,I4,1X,I4,3X,I4,1X,I4,1X,I4,2X,I3,1X,I3,1X,I3)
 4000 FORMAT( '-- PBS3CHECK TEST SET (', I4, ' ) IS COMPLETED --')
 5000 FORMAT( /, '== THERE WERE FAILURES IN THIS RUN ==')
 6000 FORMAT( /, '== THERE WERE NO FAILURES IN (', I4, ' ) ',
     $        'TEST SETS ==')
 7000 FORMAT( /, 'End of PBS3CHECK TESTS.' )
*
      STOP
*
*     End of PBS3CHECK
*
      END
