      SUBROUTINE PCVECCHK( ICONTXT, XDIST, N, NB, NZ, X, INCX, Y, INCY,
     $                     IXROW,  IXCOL, ERROR )
*
*  -- PB-BLAS auxiliary routine (version 2.0) --
*     Oak Ridge National Laboratory and University of Tennessee
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      CHARACTER*1          XDIST
      INTEGER              ICONTXT, N, NB, NZ, INCX, INCY, IXROW, IXCOL
      REAL                 ERROR
*     ..
*     .. Array Arguments ..
      COMPLEX              X( * ), Y( * )
*     ..
*
*  Purpose
*  =======
*
*  PBCVECCHK : Parallel Complex Single precision VECtor ReSulT CHecKer.
*  X is computed parallelly (by PB-BLAS, ScaLAPACK routines) and
*  Y is computed sequentially (by BLAS, LAPACK routines).
*
*  Arguments
*  =========
*
*  ICONTXT  (input) INTEGER
*           ICONTXT is the BLACS mechanism for partitioning communication
*           space.  A defining property of a context is that a message in
*           a context cannot be sent or received in another context.  The
*           BLACS context includes the definition of a grid, and each
*           process' coordinates in it.
*
*  XDIST    (input) CHARACTER*1
*           XDIST specifies the distribution of vector x as follows:
*
*             XDIST = 'C' or 'c'   x is distributed columnwise
*                                  or in a column of processors
*
*             XDIST = 'R' or 'r'   x is distributed rowwise
**                                   or in a row of processors
*  N        (input) INTEGER
*           N specifies the (global) number of the vectors X and Y.
*           N >= 0.
*
*  NB       (input) INTEGER
*           NB specifies the block size of vectors X and Y. NB >= 1.
*
*  NZ       (input) INTEGER
*           NZ is the row offset number to specify the row distance from
*           from the beginning of the block to the first element of X if
*           XDIST = 'C'.  Otherwise, it is column offset number to
*           specify the column distance from the beginning of the block
*           to the first element of X.  0 <= NZ < NB.
*
*
*  INCX     (input) INTEGER
*           INCX specifies the increment for the elements of X.
*           INCX <> 0.
*
*
*  INCY     (input) INTEGER
*           INCY specifies the increment for the elements of Y.
*           INCY <> 0.
*
*  IXROW    (input) INTEGER
*           Row processor which holds the first element of the vector X.
*
*  IXCOL    (input) INTEGER
*           Column processor which holds the first element of the vector X.
*
*           If Error = ZERO, the parallelly computed results are
*           correct.  Otherwise, the results are not correct.
*
*  =====================================================================
*
*     ..
*     .. Parameters ..
      REAL               ZERO
      PARAMETER          ( ZERO = 0.0E+0 )
*     ..
*     .. Local Scalars ..
      LOGICAL            COLUMN
      INTEGER            NPROW, NPCOL, MYROW, MYCOL, MRROW, MRCOL
      INTEGER            NP, NQ, NG, NL, I1, I2, IT, IX, IY, KZ, INFO
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, NUMROC, ICAMAX
      EXTERNAL           LSAME, ICEIL, NUMROC, ICAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           PXERBLA, SGAMX2D, BLACS_GRIDINFO
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MOD, ABS
*
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     .. Check input parameters ..
*
      COLUMN = LSAME( XDIST, 'C' )
*
      INFO = 0
      IF( ( .NOT.COLUMN             ).AND.
     $    ( .NOT.LSAME( XDIST, 'R') )         ) THEN
         INFO = 2
      ELSE IF( N    .LT.0                     ) THEN
         INFO = 3
      ELSE IF( NB   .LT.1                     ) THEN
         INFO = 4
      ELSE IF( NZ.LT.0 .AND. NZ.GT.NB         ) THEN
         INFO = 5
      ELSE IF( INCX .LT.0                     ) THEN
         INFO = 7
      ELSE IF( INCY .LT.0                     ) THEN
         INFO = 9
      ELSE IF( IXROW.LT.0 .OR. IXROW.GE.NPROW ) THEN
         INFO = 10
      ELSE IF( IXCOL.LT.0 .OR. IXCOL.GE.NPCOL ) THEN
         INFO = 11
      END IF
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICONTXT, 'PCVECCHK ', INFO )
         RETURN
      END IF
*     ..
*     .. Executable Statements ..
*
      MRROW = MOD( NPROW+MYROW-IXROW, NPROW )
      MRCOL = MOD( NPCOL+MYCOL-IXCOL, NPCOL )
      ERROR = ZERO
      NG = ( N-1 ) * INCY + 1
      KZ = 0
*
      IF( COLUMN ) THEN
         NP = NUMROC( N+NZ, NB, MRROW, 0, NPROW )
         IF( MRROW.EQ.0 ) THEN
            NP = NP - NZ
            KZ = NZ
         END IF
         NL = ( NP-1 ) * INCX + 1
*
         IF( MRCOL.EQ.0 .AND. NP.GT.0 ) THEN
            IX = 1
            IY = MAX( 0, MRROW*NB-NZ ) + 1
            DO 20 I2 = 1, ICEIL( NP+KZ, NB )
               DO 10 I1 = 1, NB-KZ
                  IF( IX.GT.NL .OR. IY.GT.NG ) GO TO 30
                  X( IX ) = X( IX ) - Y( IY )
                  IX = IX + INCX
                  IY = IY + INCY
   10          CONTINUE
               KZ = 0
               IY = IY + (NPROW-1)*NB
   20       CONTINUE
*
   30       CONTINUE
            IT = ICAMAX( NP, X, INCX )
            ERROR = ABS( X( ( IT-1 ) * INCX + 1 ) )
         END IF
         GO TO 70
*
      ELSE
         NQ = NUMROC( N+NZ, NB, MRCOL, 0, NPCOL )
         IF( MRCOL.EQ.0 ) THEN
            NQ = NQ - NZ
            KZ = NZ
         END IF
         NL = ( NQ-1 ) * INCX + 1
*
         IF( MRROW.EQ.0 .AND. NQ.GT.0 ) THEN
            IX = 1
            IY = MAX( 0, MRCOL*NB-NZ ) + 1
            DO 50 I2 = 1, ICEIL( NQ+KZ, NB )
               DO 40 I1 = 1, NB-KZ
                  IF( IX.GT.NL .OR. IY.GT.NG ) GO TO 60
                  X( IX ) = X( IX ) - Y( IY )
                  IX = IX + INCX
                  IY = IY + INCY
   40          CONTINUE
               KZ = 0
               IY = IY + ( NPCOL-1 ) * NB
   50       CONTINUE
*
   60       IT = ICAMAX( NQ, X, INCX )
            ERROR = ABS( X( ( IT-1 ) * INCX + 1 ) )
         END IF
      END IF
*
   70 CONTINUE
      CALL SGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, ERROR, 1, I1, I2, 1,
     $              -1, -1 )
*
      RETURN
*
*     END of PCVECCHK
*
      END
