      SUBROUTINE PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ, A, LDA, B,
     $                     LDB, IAROW, IACOL, EPS, ERROR )
*
*  -- PB-BLAS auxiliary routine (version 2.0) --
*     Oak Ridge National Laboratory and University of Tennessee
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      INTEGER              M, N, MB, NB, MZ, NZ, LDA, LDB
      INTEGER              ICONTXT, IAROW, IACOL
      DOUBLE PRECISION     EPS, ERROR
*     ..
*     .. Array Arguments ..
      COMPLEX*16           A( LDA, * ), B( LDB, * )
*     ..
*
*  Purpose
*  =======
*
*  PBZMT2CHK : Parallel Complex Double precision MaTriX 2 ReSulT CHecKer.
*  A is computed parallelly (by PB-BLAS, ScaLAPACK routines) and
*  B is computed sequentially (by BLAS, LAPACK routines).
*
*  Arguments
*  =========
*
*  ICONTXT  (input) INTEGER
*           ICONTXT is the BLACS mechanism for partitioning communication
*           space.  A defining property of a context is that a message in
*           a context cannot be sent or received in another context.  The
*           BLACS context includes the definition of a grid, and each
*           process' coordinates in it.
*
*  M        (input) INTEGER
*           Number of rows of the global matrices A and B.
*
*  N        (input) INTEGER
*           Number of columns of the global matrices A and B.
*
*  MB       (input) INTEGER
*           The row block size of A.
*
*  NB       (input) INTEGER
*           The column block size of A.
*
*  MZ       (input) INTEGER
*           MZ is the row offset number to specify the row distance
*           from the beginning of the block to the first element of A.
*           0 <= MZ < MB.
*
*  NZ       (input) INTEGER
*           NZ is the column offset number to specify the column
*           distance from the beginning of the block to the first
*           element of A.  0 <= NZ < NB.
*
*  A        (input/output) COMPLEX*16, dimension ( LDA, Nq )
*           The pointer to the local piece of the global matrix A,
*           which was computed in parallel.
*
*  LDA      (input) INTEGER
*           Leading Dimension of A.  LDA >= MAX(1,Mp).
*
*  B        (output) COMPLEX*16, dimension ( LDB, N )
*           The pointer to the global matrix B, which was computed
*           sequentially.
*
*  LDB      (input) INTEGER
*           Leading Dimension of B.  LDB >= MAX(1,M).
*
*  IAROW    (input) INTEGER
*           Row processor which holds the first block of the matrix A.
*
*  IACOL    (input) INTEGER
*           Column processor which holds the first block of the matrix
*           A.
*
*  EPS      (input) DOUBLE PRECISION
*           Endurable (tolerable) error given by user.
*
*  ERROR    (output) DOUBLE PRECISION
*           If ERROR <= EPS, the parallelly computed results are
*           correct.  Otherwise, the results are not correct.
*
*  =====================================================================
*
*     ..
*     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER          ( ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            NPROW, NPCOL, MYROW, MYCOL, MRROW, MRCOL
      INTEGER            MP, NQ, I1, I2, J1, J2, INFO
      INTEGER            IP, IS, JP, JS, IX, IZ, JZ
*     ..
*     .. External Functions ..
      INTEGER            ICEIL, NUMROC, IZAMAX
      EXTERNAL           ICEIL, NUMROC, IZAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           PXERBLA, DGAMX2D, BLACS_GRIDINFO
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MOD, ABS
*
      CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     .. Check input parameters ..
*
      INFO = 0
      IF(      M    .LT.0                     ) THEN
         INFO = 2
      ELSE IF( N    .LT.0                     ) THEN
         INFO = 3
      ELSE IF( MB   .LT.1                     ) THEN
         INFO = 4
      ELSE IF( NB   .LT.1                     ) THEN
         INFO = 5
      ELSE IF( MZ.LT.0 .OR. MZ.GE.MB          ) THEN
         INFO = 6
      ELSE IF( NZ.LT.0 .OR. NZ.GE.NB          ) THEN
         INFO = 7
      ELSE IF( LDB  .LT.M                     ) THEN
         INFO = 11
      ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
         INFO = 12
      ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
         INFO = 13
      END IF
*
   10 CONTINUE
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICONTXT, 'PZMT2CHK ', INFO )
         RETURN
      END IF
*     ..
*     .. Executable Statements ..
*
      MRROW = MOD( NPROW+MYROW-IAROW, NPROW )
      MRCOL = MOD( NPCOL+MYCOL-IACOL, NPCOL )
*
      MP = NUMROC( M+MZ, MB, MRROW, 0, NPROW )
      IF( MRROW.EQ.0 ) MP = MP - MZ
*
      JZ = 0
      NQ = NUMROC( N+NZ, NB, MRCOL, 0, NPCOL )
      IF( MRCOL.EQ.0 ) THEN
        NQ = NQ - NZ
        JZ = NZ
      END IF
      ERROR = ZERO
*
      IF( LDA.LT.MP ) THEN
        INFO = 9
        GO TO 10
      END IF
*
      IF( MP.EQ.0 .OR. NQ.EQ.0 ) GO TO 70
*
      JP = 0
      JS = MAX( 0, MRCOL*NB-NZ )
      IZ = 0
*
      DO 60 J2 = 1, ICEIL( NQ+JZ, NB )
         DO 50 J1 = 1, NB-JZ
            IF( JP+J1.GT.NQ .OR. JS+J1.GT.N ) GO TO 70
            IP = 0
            IS = MAX( 0, MRROW*MB-MZ )
            IF( MRROW.EQ.0 ) IZ = MZ
*
            DO 30 I2 = 1, ICEIL( MP+IZ, MB )
               DO 20 I1 = 1, MB-IZ
                  IF( IP+I1.GT.MP .OR. IS+I1.GT.M ) GO TO 40
                  A( IP+I1, JP+J1 ) = A( IP+I1, JP+J1 ) -
     $                                B( IS+I1, JS+J1 )
   20          CONTINUE
               IP = IP + MB - IZ
               IS = IS + NPROW*MB - IZ
               IZ = 0
   30       CONTINUE
*
   40       CONTINUE
            IX = IZAMAX( MP, A( 1, JP+J1 ), 1 )
            ERROR = MAX( ERROR, ABS( A( IX, JP+J1 ) ) )
            IF( ERROR.GT.EPS ) GO TO 70
   50    CONTINUE
*
         JP = JP + NB - JZ
         JS = JS + NPCOL*NB - JZ
         JZ = 0
   60 CONTINUE
*
   70 CONTINUE
      CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, ERROR, 1, I1, I2, 1,
     $              -1, -1 )
*
      RETURN
*
*     END of PZMT2CHK
*
      END
