


/* $Id: PDMAP.base,v 1.36 1994/11/28 18:08:36 lprylli Exp $
 * ------------------------------------------------------------------------
 * 
 * -- ScaLAPACK routine (version 2.0) -- Oak Ridge National Laboratory, Univ. of
 * Tennessee, and Univ. of California, Berkeley. October 31, 1994.
 * 
 * SUBROUTINE PZGEMR2D( M, N, $                     A, IA, JA, ADESC, $
 * B, IB, JB, BDESC)
 * ------------------------------------------------------------------------
 * Purpose =======
 * 
 * PZGEMR2D copies a submatrix of A on a submatrix of B. A and B can have
 * different distributions: they can be on different processor grids, they
 * can have different blocksizes, the beginning of the area to be copied can
 * be at a different places on A and B.
 * 
 * 
 * Notes =====
 * 
 * A description vector is associated with each 2D block-cyclicly dis- tributed
 * matrix.  This vector stores the information required to establish the
 * mapping between a matrix entry and its corresponding process and memory
 * location.
 * 
 * In the following comments, the character _ should be read as "of the
 * distributed matrix".  Let A be a generic term for any 2D block cyclicly
 * distributed matrix.  Its description vector is DESC_A:
 * 
 * NOTATION  STORED IN   EXPLANATION --------  ---------
 * ----------------------------------------------- M_A       desc_A[0]
 * Number of rows in the distributed matrix. N_A       desc_A[1]   Number of
 * columns in the distributed matrix. MB_A      desc_A[2]   The blocking
 * factor used to distribute the rows of the matrix. NB_A      desc_A[3]
 * The blocking factor used to distribute the columns of the matrix. RSRC_A
 * desc_A[4]   The process row over which the first row of the matrix is
 * distributed. CSRC_A    desc_A[5]   The process column over which the first
 * column of the matrix is distributed. CTXT_A    desc_A[6]   A BLACS context
 * handle, indicating the context where the matrix is defined. LLD_A
 * desc_A[7]   The leading dimension of the local array storing the process's
 * portion of the matrix A. LLD_A >= MAX( 1, LOCp(M_A) ).
 * 
 * 
 * Restrictions of the present PBLAS version
 * =========================================
 * 
 * All nodes must call the redistribute routine, even those that are neither in
 * the first grid nor in the second.
 * 
 * Parameters ==========
 * 
 * 
 * M        (input) INTEGER. On entry, M specifies the number of rows of the
 * submatrix to be copied.  M must be at least zero. Unchanged on exit.
 * 
 * N        (input) INTEGER. On entry, N specifies the number of cols of the
 * submatrix to be redistributed.rows of B.  M must be at least zero.
 * Unchanged on exit.
 * 
 * A        (input) COMPLEX*16 On entry, the source matrix. Unchanged on exit.
 * 
 * IA,JA    (input) INTEGER On entry,the coordinates of the beginning of the
 * submatrix of A to copy. 1 <= IA <= M_A - M + 1,1 <= JA <= N_A - N + 1,
 * Unchanged on exit.
 * 
 * ADESC    (input) A description vector (see Notes above)
 * 
 * 
 * B        (output) COMPLEX*16 On entry, the destination matrix. The portion
 * corresponding to the defined submatrix are updated.
 * 
 * IB,JB    (input) INTEGER On entry,the coordinates of the beginning of the
 * submatrix of B that will be updated. 1 <= IB <= M_B - M + 1,1 <= JB <= N_B
 * - N + 1, Unchanged on exit.
 * 
 * BDESC    (input) B description vector (see Notes above)
 * 
 * 
 * 
 * Memory requirement : ====================
 * 
 * for the processors belonging to grid 0, one buffer of size block 0 and for
 * the processors belonging to grid 1, also one buffer of size block 1.
 * 
 * ============================================================ Created March
 * 1993 by B. Tourancheau (See sccs for modifications).
 * ============================================================ */






#ifdef Add_
#define fortran_mr2d pzgemr2d_
#elif defined(Upcase)
#define fortran_mr2d PZGEMR2D
#define zcopy_ ZCOPY
#define zlacpy_ ZLACPY
#else
#define fortran_mr2d pzgemr2d
#define zcopy_ zcopy
#define zlacpy_ zlacpy
#endif

#define Clacpy Czgelacpy
void  Clacpy();
typedef struct {
  double r, i;
}     dcomplex;

typedef struct {
  int   m;
  int   n;
  int   nbrow;
  int   nbcol;
  int   sprow;
  int   spcol;
  int   ctxt;
  int   lda;
}     MDESC;

typedef struct {
  int   lstart;
  int   len;
}     IDESC;
#define SHIFT(row,sprow,nbrow) ((row)-(sprow)+ ((row) >= (sprow) ? 0 : (nbrow)))

#define max(A,B) ((A)>(B)?(A):(B))
#define min(A,B) ((A)>(B)?(B):(A))
#define DIVUP(a,b) ( ((a)-1) /(b)+1)
#define ROUNDUP(a,b) (DIVUP(a,b)*(b))

#ifdef MALLOCDEBUG
#define malloc mymalloc
#define free myfree
#define realloc myrealloc
#endif



/* Cblacs */
extern void Cblacs_pcoord();
extern int Cblacs_pnum();
extern void Csetpvmtids();
extern void Cblacs_pinfo();
extern void Cblacs_gridinfo();
extern void Cblacs_gridinit();
extern void Cblacs_exit();
extern void Cblacs_gridexit();
extern void Cblacs_setup();
extern void Cigebs2d();
extern void Cigebr2d();
extern void Cigesd2d();
extern void Cigerv2d();
extern void Cigsum2d();
extern void Cigamn2d();
extern void Cigamx2d();
extern void Czgesd2d();
extern void Czgerv2d();


/* lapack */
void  zlacpy_();

/* aux fonctions */
extern int localindice();
extern void *mr2d_malloc();
extern int ppcm();
extern int localsize();
extern int memoryblocksize();
extern int changeorigin();
extern void paramcheck();

/* tools and others function */
#define scanD0 zgescanD0
#define dispmat zgedispmat
#define setmemory zgesetmemory
#define freememory zgefreememory
#define scan_intervals zgescan_intervals


extern void scanD0();
extern void dispmat();
extern void setmemory();
extern void freememory();
extern int scan_intervals();
extern void Cpzgemr2d();

/* some defines for Cpzgemr2d */
#define SENDBUFF 0
#define RECVBUFF 1
#define SIZEBUFF 2

#ifndef DEBUG
#define NDEBUG
#endif



#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


#define DESCLEN 8
void 
fortran_mr2d(m, n, A, ia, ja, desc_A,
	     B, ib, jb, desc_B)
  int  *ia, *ib, *ja, *jb, *m, *n;
  int   desc_A[DESCLEN], desc_B[DESCLEN];
  dcomplex *A, *B;
{
  Cpzgemr2d(*m, *n, A, *ia, *ja, (MDESC *) desc_A,
	    B, *ib, *jb, (MDESC *) desc_B);
  return;
}



static void init_chenille();
static int inter_len();
static int block2buff();
static void buff2block();


void
Cpzgemr2d(m, n,
	  ptrmyblock, ia, ja, ma,
	  ptrmynewblock, ib, jb, mb)
  dcomplex *ptrmyblock, *ptrmynewblock;
/* pointers to the memory location of the matrix and the redistributed matrix */
  MDESC *ma;
  MDESC *mb;
  int   ia, ja, ib, jb, m, n;

{
  dcomplex *ptrsendbuff, *ptrrecvbuff, *ptrNULL = 0;
  MDESC newa, newb;
  int  *proc0, *proc1, *param;
  int   mypnum, myprow0, mypcol0, myprow1, mypcol1, nprocs;
  int   i, j;
  int   recvsize, sendsize;
  int   gcontext;
  IDESC *h_inter;	/* to store the horizontal intersections */
  IDESC *v_inter;	/* to store the vertical intersections */
  int   hinter_nb, vinter_nb;	/* number of intrsections in both directions */
  int   dummy;
  int   p0, q0, p1, q1;

  /* end of variables */

  /* To simplify further calcul we change the matrix indexation from
   * 1..m,1..n (fortran) to 0..m-1,0..n-1 */
  ia -= 1;
  ja -= 1;
  ib -= 1;
  jb -= 1;
  /* first we initialize a global grid which serve as a reference to
   * communicate from grid a to grid b */
  Cblacs_pinfo(&dummy, &nprocs);
  Cblacs_gridinit(&gcontext, "R", 1, nprocs);
  Cblacs_gridinfo(gcontext, &dummy, &dummy, &dummy, &mypnum);
  Cblacs_gridinfo(ma->ctxt, &p0, &q0, &myprow0, &mypcol0);
  assert((myprow0 < p0 && mypcol0 < q0) || (myprow0 == -1 && mypcol0 == -1));
  Cblacs_gridinfo(mb->ctxt, &p1, &q1, &myprow1, &mypcol1);
  assert((myprow1 < p1 && mypcol1 < q1) || (myprow1 == -1 && mypcol1 == -1));

  /* exchange the missing parameters among the processors: shape of grids and
   * location of the processors */
  param = (int *) mr2d_malloc((nprocs * 2 + 4) * sizeof(int));
  for (i = 0; i < nprocs * 2 + 4; i++)
    param[i] = nprocs + 10000;
  proc0 = param + 4;
  proc1 = param + 4 + nprocs;

  /* we calulate proc0 and proc1 that will give the number of a proc in
   * respectively a or b in the global context */
  if (myprow0 >= 0) {
    proc0[myprow0 * q0 + mypcol0] = mypnum;
    param[0] = p0;
    param[1] = q0;
  }
  if (myprow1 >= 0) {
    proc1[myprow1 * q1 + mypcol1] = mypnum;
    param[2] = p1;
    param[3] = q1;
  }
  Cigamn2d(gcontext, "All", "H", 2 * nprocs + 4, 1, param, 2 * nprocs + 4, NULL, NULL, -1, -1, -1);
  p0 = param[0];
  q0 = param[1];
  p1 = param[2];
  q1 = param[3];
#ifndef NDEBUG
  for (i = 0; i < p0 * q0; i++)
    assert(proc0[i] >= 0 && proc0[i] < nprocs);
  for (i = 0; i < p1 * q1; i++)
    assert(proc1[i] >= 0 && proc1[i] < nprocs);
#endif

  /* check the validity of the parameters */
  paramcheck(ma, ia, ja, m, n, p0, q0);
  paramcheck(mb, ib, jb, m, n, p1, q1);


  /* we change the problem so that ia < a->nbrow ... andia + m = a->m ... */
  {
    int   decal;
    newa = *ma;
    newb = *mb;
    ma = &newa;
    mb = &newb;
    ia = changeorigin(myprow0, ma->sprow, p0,
		      ma->nbrow, ia, &decal, &ma->sprow);
    ptrmyblock += decal;
    ja = changeorigin(mypcol0, ma->spcol, q0,
		      ma->nbcol, ja, &decal, &ma->spcol);
    ptrmyblock += decal * ma->lda;
    ma->m = ia + m;
    ma->n = ja + n;
    ib = changeorigin(myprow1, mb->sprow, p1,
		      mb->nbrow, ib, &decal, &mb->sprow);
    ptrmynewblock += decal;
    jb = changeorigin(mypcol1, mb->spcol, q1,
		      mb->nbcol, jb, &decal, &mb->spcol);
    ptrmynewblock += decal * mb->lda;
    mb->m = ib + m;
    mb->n = jb + n;
#ifndef NDEBUG
    paramcheck(ma, ia, ja, m, n, p0, q0);
    paramcheck(mb, ib, jb, m, n, p1, q1);
#endif
  }



  /* We compute the size of the memory buffer ( we choose the worst case,
   * when the buffer sizes == the memory block sizes). */
  if (myprow0 >= 0 && mypcol0 >= 0) {
    /* Initialize pointer variables */
    setmemory(&ptrsendbuff, memoryblocksize(ma));
  };	/* if (mypnum < p0 * q0) */

  if (myprow1 >= 0 && mypcol1 >= 0) {
    /* Initialize pointer variables */
    setmemory(&ptrrecvbuff, memoryblocksize(mb));
  };	/* if (mypnum < p1 * q1) */


  /* allocing room for the tabs, alloc for the worst case,local_n or local_m
   * intervals, in fact the worst case should be less, perhaps half that,I
   * should think of that one day. */
  h_inter = (IDESC *) mr2d_malloc(DIVUP(ma->n, q0 * ma->nbcol) *
				  ma->nbcol * sizeof(IDESC));
  v_inter = (IDESC *) mr2d_malloc(DIVUP(ma->m, p0 * ma->nbrow)
				  * ma->nbrow * sizeof(IDESC));
  /* We go for the scanning of indices. For each processor including mypnum,
   * we fill the sendbuff buffer (scanD0(SENDBUFF)) and when it is done send
   * it. Then for each processor, we compute the size of message to be
   * receive scanD0(SIZEBUFF)), post a receive and then allocate the elements
   * of recvbuff the right place (scanD)(RECVBUFF)) */
  {
    int   tot, myrang, step, sens;
    int  *sender, *recver;
    tot = max(p0 * q0, p1 * q1);
    init_chenille(mypnum, nprocs, p0 * q0, proc0, p1 * q1, proc1,
		  &sender, &recver, &myrang);
    if (myrang == -1)
      goto after_comm;
    step = tot - 1 - myrang;
    do {
      for (sens = 0; sens < 2; sens++) {
	/* be careful here, when we communicating with ourselves, we must
	 * send first (myrang > step == 0) */
	if (sender[myrang] >= 0 && recver[step] >= 0 &&
	    (sens == 0)) {
	  i = recver[step] / q1;
	  j = recver[step] % q1;
	  vinter_nb = scan_intervals('r', ia, ib, m, ma, mb, p0, p1, myprow0, i,
				     v_inter);
	  hinter_nb = scan_intervals('c', ja, jb, n, ma, mb, q0, q1, mypcol0, j,
				     h_inter);
	  sendsize = block2buff(v_inter, vinter_nb, h_inter, hinter_nb,
				ptrmyblock, ma, ptrsendbuff);
	}	/* if (sender[myrang] >=0...) { */
	if (sender[myrang] >= 0 && recver[step] >= 0 &&
	    (sens == myrang > step)) {
	  i = recver[step] / q1;
	  j = recver[step] % q1;

	  if (sendsize > 0
	      && proc0[sender[myrang]] != proc1[recver[step]]
		) {
	    Czgesd2d(gcontext, sendsize, 1, ptrsendbuff, sendsize,
		     0, proc1[i * q1 + j]);
	  }	/* sendsize > 0 */
	}	/* if (sender [myrang >= 0) ... */
	if (recver[myrang] >= 0 && sender[step] >= 0 &&
	    (sens == myrang <= step)) {
	  i = sender[step] / q0;
	  j = sender[step] % q0;
	  vinter_nb = scan_intervals('r', ib, ia, m, mb, ma, p1, p0, myprow1, i,
				     v_inter);
	  hinter_nb = scan_intervals('c', jb, ja, n, mb, ma, q1, q0, mypcol1, j,
				     h_inter);
	  recvsize = inter_len(hinter_nb, h_inter, vinter_nb, v_inter);
	  if (recvsize > 0) {
	    if (proc0[sender[step]] == proc1[recver[myrang]]) {
	      Clacpy(recvsize, 1,
		     ptrsendbuff, recvsize,
		     ptrrecvbuff, recvsize);
	    } else {
	      Czgerv2d(gcontext, recvsize, 1, ptrrecvbuff, recvsize,
		       0, proc0[i * q0 + j]);
	    }
	  }	/* recvsize > 0 */
	}	/* if (recver[myrang] ...) */
	if (recver[myrang] >= 0 && sender[step] >= 0 && sens == 1) {
	  buff2block(v_inter, vinter_nb, h_inter, hinter_nb,
		     ptrrecvbuff, ptrmynewblock, mb);
	}	/* if (recver[myrang] >=0...)  */
      }	/* for (sens = 0) */
      step -= 1;
      if (step < 0)
	step = tot - 1;
    } while (step != tot - 1 - myrang);
after_comm:
    free(sender);
  }	/* { int tot,nr,ns ...} */


  /* don't forget to clean up things! */
  if (myprow1 >= 0 && mypcol1 >= 0) {
    freememory((char *) ptrrecvbuff);
  };
  if (myprow0 >= 0 && mypcol0 >= 0) {
    freememory((char *) ptrsendbuff);
  };
  Cblacs_gridexit(gcontext);
  free(v_inter);
  free(h_inter);
  free(param);
}/* distrib */



static void 
init_chenille(mypnum, nprocs, n0, proc0, n1, proc1, psend, precv, myrang)
  int   nprocs, mypnum, n0, n1;
  int  *proc0, *proc1, **psend, **precv, *myrang;
{
  int   ns, nr, i, tot;
  int  *sender, *recver, *g0, *g1;
  tot = max(n0, n1);
  sender = (int *) mr2d_malloc((nprocs + tot) * sizeof(int) * 2);
  recver = sender + tot;
  *psend = sender;
  *precv = recver;
  g0 = recver + tot;
  g1 = g0 + nprocs;
  for (i = 0; i < nprocs; i++) {
    g0[i] = -1;
    g1[i] = -1;
  }
  for (i = 0; i < tot; i++) {
    sender[i] = -1;
    recver[i] = -1;
  }
  for (i = 0; i < n0; i++)
    g0[proc0[i]] = i;
  for (i = 0; i < n1; i++)
    g1[proc1[i]] = i;
  ns = 0;
  nr = 0;
  *myrang = -1;
  for (i = 0; i < nprocs; i++)
    if (g0[i] >= 0 && g1[i] >= 0) {
      if (i == mypnum)
	*myrang = nr;
      sender[ns] = g0[i];
      ns += 1;
      recver[nr] = g1[i];
      nr += 1;
      assert(ns <= n0 && nr <= n1 && nr == ns);
    }
  for (i = 0; i < nprocs; i++)
    if (g0[i] >= 0 && g1[i] < 0) {
      if (i == mypnum)
	*myrang = ns;
      sender[ns] = g0[i];
      ns += 1;
      assert(ns <= n0);
    }
  for (i = 0; i < nprocs; i++)
    if (g1[i] >= 0 && g0[i] < 0) {
      if (i == mypnum)
	*myrang = nr;
      recver[nr] = g1[i];
      nr += 1;
      assert(nr <= n1);
    }
}

static int 
block2buff(vi, vinb, hi, hinb, ptra, ma, buff)
  int   hinb, vinb;
  IDESC *hi, *vi;
  MDESC *ma;
  dcomplex *buff, *ptra;
{
  int   h, v, sizebuff;
  dcomplex *ptr2;
  sizebuff = 0;
  for (h = 0; h < hinb; h++) {
    ptr2 = ptra + hi[h].lstart * ma->lda;
    for (v = 0; v < vinb; v++) {
      Clacpy(vi[v].len, hi[h].len,
	     ptr2 + vi[v].lstart,
	     ma->lda,
	     buff + sizebuff, vi[v].len);
      sizebuff += hi[h].len * vi[v].len;
    }
  }
  return sizebuff;
}

static void 
buff2block(vi, vinb, hi, hinb, buff, ptrb, mb)
  int   hinb, vinb;
  IDESC *hi, *vi;
  MDESC *mb;
  dcomplex *buff, *ptrb;
{
  int   h, v, sizebuff;
  dcomplex *ptr2;
  sizebuff = 0;
  for (h = 0; h < hinb; h++) {
    ptr2 = ptrb + hi[h].lstart * mb->lda;
    for (v = 0; v < vinb; v++) {
      Clacpy(vi[v].len, hi[h].len,
	     buff + sizebuff, vi[v].len,
	     ptr2 + vi[v].lstart,
	     mb->lda);
      sizebuff += hi[h].len * vi[v].len;
    }
  }
}


static int 
inter_len(hinb, hi, vinb, vi)
  int   hinb, vinb;
  IDESC *hi, *vi;
{
  int   hlen, vlen, h, v;
  hlen = 0;
  for (h = 0; h < hinb; h++)
    hlen += hi[h].len;
  vlen = 0;
  for (v = 0; v < vinb; v++)
    vlen += vi[v].len;
  return hlen * vlen;
}


void 
Clacpy(m, n, a, lda, b, ldb)
  dcomplex *a, *b;
  int   m, n, lda, ldb;
{
  int   i, j;
  lda -= m;
  ldb -= m;
  assert(lda >= 0 && ldb >= 0);
  for (j = 0; j < n; j++) {
    for (i = 0; i < m; i++)
      *b++ = *a++;
    b += ldb;
    a += lda;
  }
}
