      SUBROUTINE PDSEPCHK( MS, NV, A, EPSNORMA, THRESH, Q, IQ, JQ,
     $                     DESCQ, B, IA, JA, DESCA, W, WORK, LWORK,
     $                     TSTNRM, RESULT )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IA, IQ, JA, JQ, LWORK, MS, NV, RESULT
      DOUBLE PRECISION   EPSNORMA, THRESH, TSTNRM
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( 8 ), DESCQ( 8 )
      DOUBLE PRECISION   A( * ), B( * ), Q( * ), W( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  Compute |AQ -QL| / (abstol + eps * norm(A) )
*
*
*  Notes
*  =====
*
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             global context of the operation on the
*                             matrix. The context itself is global.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*     MP = number of local rows in A, B and Q
*     MQ = number of local columns in A
*     NQ = number of local columns in B and Q
*
*  MS      (global input) INTEGER
*          Matrix size.
*          The number of global rows in A and B and Q
*          Also, the number of columns in A
*
*  NV      (global input) INTEGER
*          Number of eigenvectors
*          The number of global columns in B and Q.
*
*  A       (input) block cyclic distributed DOUBLE PRECISION array,
*          global dimension (MS, MS), local dimension (DESCA(8), MQ)
*
*          Contains the original test matrix
*
*  EPSNORMA (input) DOUBLE PRECISION
*          eps * norm(A)
*
*  THRESH  (input) DOUBLE PRECISION
*          A test will count as "failed" if the "error", computed as
*          described below, exceeds THRESH.  Note that the error
*          is scaled to be O(1), so THRESH should be a reasonably
*          small multiple of 1, e.g., 10 or 100.  In particular,
*          it should not depend on the precision (single vs. double)
*          or the size of the matrix.  It must be at least zero.
*
*  Q       (local input) DOUBLE PRECISION array
*          global dimension (MS, NV), local dimension (DESCA(8), NQ)
*
*          Contains the eigenvectors as computed by PDSYEVX
*
*
*  IQ      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix Q to operate on.
*
*
*  JQ      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix Q to operate on.
*
*
*  DESCQ   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix Q.
*
*  B       (local workspace) DOUBLE PRECISION array,
*          global dimension (MS, NV), local dimension (DESCA(8), MQ)
*
*          Accumulator for computing AQ -QL
*
*
*  IA      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  JA      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  W       (global input) Double Precision array, dimension (NV)
*
*          Contains the computed eigenvalues
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*
*  LWORK   (local input) INTEGER
*          The length of the array WORK.
*          LWORK >= NUMROC( NV, DESCA( 4 ), MYCOL, 0, NPCOL )
*
*  TSTNRM  (global output) DOUBLE PRECISION
*          |AQ- QL| / |A|*EPS
*
*  RESULT  (global output) INTEGER
*          0 if the test passes i.e.
*            |AQ -QL| / (abstol + eps * norm(A) ) <= n* THRESH
*          1 if the test fails  i.e.
*            |AQ -QL| / (abstol + eps * norm(A) ) > n * THRESH
*     .. Local Scalars ..
*
      INTEGER            INFO, J, LOCALCOL, MP, MYCOL, MYROW, NPCOL,
     $                   NPROW, NQ, PCOL
      DOUBLE PRECISION   NORM
*     ..
*     .. Parameters ..
      DOUBLE PRECISION   ONE, NEGONE
      PARAMETER          ( ONE = 1.0D0, NEGONE = -1.0D0 )
*     ..
*     .. External Functions ..
      INTEGER            INDXG2L, INDXG2P, NUMROC
      DOUBLE PRECISION   PDLANGE
      EXTERNAL           INDXG2L, INDXG2P, NUMROC, PDLANGE
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DLACPY, DSCAL, PDGEMM,
     $                   PXERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
      RESULT = 0
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
*
      INFO = 0
      CALL CHK1MAT( MS, 1, MS, 2, IQ, JQ, DESCQ, 9, INFO )
      CALL CHK1MAT( NV, 1, MS, 2, IA, JA, DESCA, 13, INFO )
*
      IF( INFO.EQ.0 ) THEN
*
         MP = NUMROC( MS, DESCA( 3 ), MYROW, 0, NPROW )
         NQ = NUMROC( NV, DESCA( 4 ), MYCOL, 0, NPCOL )
*
         IF( IQ.NE.1 ) THEN
            INFO = -7
         ELSE IF( JQ.NE.1 ) THEN
            INFO = -8
         ELSE IF( IA.NE.1 ) THEN
            INFO = -11
         ELSE IF( JA.NE.1 ) THEN
            INFO = -12
         ELSE IF( LWORK.LT.NQ ) THEN
            INFO = -16
         END IF
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( DESCA( 7 ), 'pdsepchk', -INFO )
         RETURN
      END IF
*
*     B = Q * W
*
      CALL DLACPY( 'A', MP, NQ, Q, DESCQ( 8 ), B, DESCA( 8 ) )
*
*
      DO 10 J = 1, NV
         PCOL = INDXG2P( J, DESCA( 4 ), 0, 0, NPCOL )
         LOCALCOL = INDXG2L( J, DESCA( 4 ), 0, 0, NPCOL )
*
         IF( MYCOL.EQ.PCOL ) THEN
            CALL DSCAL( MP, W( J ), B( ( LOCALCOL-1 )*DESCA( 8 )+1 ),
     $                  1 )
         END IF
   10 CONTINUE
*
*
*     B = B - A * Q
*
      CALL PDGEMM( 'N', 'N', MS, NV, MS, NEGONE, A, 1, 1, DESCA, Q, 1,
     $             1, DESCQ, ONE, B, 1, 1, DESCA )
*
*     Compute the norm of B
*
*
      NORM = PDLANGE( '1', MS, NV, B, 1, 1, DESCA, WORK )
*
*
      TSTNRM = NORM / EPSNORMA / MAX( MS, 1 )
*
      IF( TSTNRM.GT.THRESH )
     $   RESULT = 1
*
*
      RETURN
      END
