      SUBROUTINE PDSEPQTQ( MS, NV, THRESH, Q, IQ, JQ, DESCQ, C, IA, JA,
     $                     DESCA, PROCDIST, ICLUSTR, GAP, WORK, LWORK,
     $                     QTQNRM, INFO, RES )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IA, INFO, IQ, JA, JQ, LWORK, MS, NV, RES
      DOUBLE PRECISION   QTQNRM, THRESH
*     ..
*     .. Array Arguments ..
*
      INTEGER            DESCA( 8 ), DESCQ( 8 ), ICLUSTR( * ),
     $                   PROCDIST( * )
      DOUBLE PRECISION   C( * ), GAP( * ), Q( * ), WORK( * )
*     ..
*
*  Notes
*  =====
*
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             global context of the operation on the
*                             matrix. The context itself is global.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Purpose
*  =======
*
*  Compute |I - QT * Q| / (ulp * n)
*
*  Arguments
*  =========
*
*     NP = number of local rows in C
*     NQ = number of local columns in C and Q
*
*  MS      (global input) INTEGER
*          Matrix size.
*          The number of global rows in Q
*
*  NV      (global input) INTEGER
*          Number of eigenvectors
*          The number of global columns in C and Q
*
*  THRESH  (global input) DOUBLE PRECISION
*          A test will count as "failed" if the "error", computed as
*          described below, exceeds THRESH.  Note that the error
*          is scaled to be O(1), so THRESH should be a reasonably
*          small multiple of 1, e.g., 10 or 100.  In particular,
*          it should not depend on the precision (single vs. double)
*          or the size of the matrix.  It must be at least zero.
*
*  Q       (local input) DOUBLE PRECISION array,
*          global dimension (MS, NV), local dimension (LDQ, NQ)
*
*          Contains the eigenvectors as computed by PDSTEIN
*
*
*  IQ      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix Q to operate on.
*
*
*  JQ      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix Q to operate on.
*
*
*  DESCQ   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix Q.
*
*  C       (local workspace)  DOUBLE PRECISION array,
*          global dimension (NV, NV), local dimension (DESCA(8), NQ)
*
*          Accumulator for computing I - QT * Q
*
*
*  IC      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix C to operate on.
*
*
*  JC      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix C to operate on.
*
*
*  DESCC   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix C.
*
*  W       (input) Double Precision array, dimension (NV)
*          All procesors have an identical copy of W()
*
*          Contains the computed eigenvalues
*
*  PROCDIST (global input) INTEGER array dimensoin (NPROW*NPCOL+1)
*          Identifies which eigenvectors are the last to be computed
*          by a given process
*
*  ICLUSTR (global input) INTEGER array dimension (2*P)
*          This input array contains indices of eigenvectors
*          corresponding to a cluster of eigenvalues that could not be
*          orthogonalized due to insufficient workspace.
*          This should be the otuput of PDSTEIN.
*
*  GAP     (global input) DOUBLE PRECISION array, dimension (P)
*          This input array contains the gap between eigenvalues whose
*          eigenvectors could not be orthogonalized.
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*
*  LWORK   (local input) INTEGER
*          The length of the array WORK.
*          LWORK >= 2 + MAX( DESCA( 3 ), 2 )*( 2*NP0+NQ0 )
*          Where:
*          NP0 = NUMROC( NV, DESCA( 3 ), 0, 0, NPROW )
*          NQ0 = NUMROC( NV, DESCA( 4 ), 0, 0, NPCOL )
*
*  QTQNRM  (global output) DOUBLE PRECISION
*          |QTQ -I| / EPS
*
*  RES     (global output) INTEGER
*          0 if the test passes i.e. |I - QT * Q| / (ulp * n) <= THRESH
*          1 if the test fails  i.e. |I - QT * Q| / (ulp * n) > THRESH
*
*
*     .. Parameters ..
*
      DOUBLE PRECISION   ZERO, ONE, NEGONE
      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, NEGONE = -1.0D0 )
*     ..
*     .. Intrinsic Functions ..
*
      INTRINSIC          DBLE, MAX, MIN, MOD
*     ..
*     .. Local Scalars ..
      INTEGER            CLUSTER, FIRSTP, I, II, IIROW, IMAX, IMIN,
     $                   JJCOL, JMAX, JMIN, LII, LIII, LJJ, LJJJ, LWMIN,
     $                   MAXI, MYCOL, MYROW, NEXTP, NP, NP0, NPCOL,
     $                   NPROW, NQ, NQ0
      DOUBLE PRECISION   NORM, QTQNRM2, ULP
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      DOUBLE PRECISION   PDLAMCH, PDLANGE
      EXTERNAL           NUMROC, PDLAMCH, PDLANGE
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DLASET, PDGEMM,
     $                   PDMATADD, PXERBLA
*     ..
*     .. Executable Statements ..
*
*
      RES = 0
      ULP = PDLAMCH( DESCA( 7 ), 'P' )
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
      INFO = 0
      CALL CHK1MAT( MS, 1, MS, 2, IQ, JQ, DESCQ, 9, INFO )
      CALL CHK1MAT( NV, 1, MS, 2, IA, JA, DESCA, 13, INFO )
*
      IF( INFO.EQ.0 ) THEN
         NP0 = NUMROC( NV, DESCA( 3 ), 0, 0, NPROW )
         NQ0 = NUMROC( NV, DESCA( 4 ), 0, 0, NPCOL )
*
         LWMIN = 2 + MAX( DESCA( 3 ), 2 )*( 2*NP0+NQ0 )
*
         NP = NUMROC( NV, DESCA( 3 ), MYROW, 0, NPROW )
         NQ = NUMROC( NV, DESCA( 4 ), MYCOL, 0, NPCOL )
*
*
*
         IF( IQ.NE.1 ) THEN
            INFO = -7
         ELSE IF( JQ.NE.1 ) THEN
            INFO = -8
         ELSE IF( IA.NE.1 ) THEN
            INFO = -11
         ELSE IF( JA.NE.1 ) THEN
            INFO = -12
         ELSE IF( LWORK.LT.LWMIN ) THEN
            INFO = -16
         END IF
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( DESCA( 7 ), 'pdsepqtq', -INFO )
         RETURN
      END IF
*
*     C = Identity matrix
*
      CALL DLASET( 'A', NP, NQ, ZERO, ZERO, C, DESCA( 8 ) )
*
      IIROW = 0
      JJCOL = 0
      LII = 1
      LJJ = 1
      DO 20 II = 1, NV, DESCA( 3 )
         MAXI = MIN( MS, II+DESCA( 3 )-1 )
         IF( ( MYROW.EQ.IIROW ) .AND. ( MYCOL.EQ.JJCOL ) ) THEN
            LIII = LII
            LJJJ = LJJ
            DO 10 I = II, MAXI
               C( LIII+( LJJJ-1 )*DESCA( 8 ) ) = ONE
               LIII = LIII + 1
               LJJJ = LJJJ + 1
   10       CONTINUE
         END IF
         IF( MYROW.EQ.IIROW )
     $      LII = LII + DESCA( 3 )
         IF( MYCOL.EQ.JJCOL )
     $      LJJ = LJJ + DESCA( 3 )
         IIROW = MOD( IIROW+1, NPROW )
         JJCOL = MOD( JJCOL+1, NPCOL )
   20 CONTINUE
*
*     C = C - QT * Q
*
      IF( NV*MS.GT.0 ) THEN
         CALL PDGEMM( 'T', 'N', NV, NV, MS, NEGONE, Q, 1, 1, DESCQ, Q,
     $                1, 1, DESCQ, ONE, C, 1, 1, DESCA )
      END IF
*
*     Allow for poorly orthogonalized eigenvectors for large clusters
*
      NORM = PDLANGE( '1', NV, NV, C, 1, 1, DESCA, WORK )
      QTQNRM = NORM / ( DBLE( MAX( MS, 1 ) )*ULP )
*
      CLUSTER = 1
   30 CONTINUE
      DO 40 FIRSTP = 1, NPROW*NPCOL
         IF( PROCDIST( FIRSTP ).GE.ICLUSTR( 2*( CLUSTER-1 )+1 ) )
     $      GO TO 50
   40 CONTINUE
   50 CONTINUE
*
      IMIN = ICLUSTR( 2*CLUSTER-1 )
      JMAX = ICLUSTR( 2*CLUSTER )
*
*
      IF( IMIN.EQ.0 )
     $   GO TO 80
*
      DO 60 NEXTP = FIRSTP, NPROW*NPCOL
         IMAX = PROCDIST( NEXTP )
         JMIN = IMAX + 1
*
*
         CALL PDMATADD( IMAX-IMIN+1, JMAX-JMIN+1, ZERO, C, IMIN, JMIN,
     $                  DESCA, GAP( CLUSTER ) / 0.01D0, C, IMIN, JMIN,
     $                  DESCA )
         CALL PDMATADD( JMAX-JMIN+1, IMAX-IMIN+1, ZERO, C, JMIN, IMIN,
     $                  DESCA, GAP( CLUSTER ) / 0.01D0, C, JMIN, IMIN,
     $                  DESCA )
         IMIN = IMAX
*
         IF( ICLUSTR( 2*CLUSTER ).LT.PROCDIST( NEXTP+1 ) )
     $      GO TO 70
   60 CONTINUE
   70 CONTINUE
*
      CLUSTER = CLUSTER + 1
      GO TO 30
   80 CONTINUE
*
*     Compute the norm of C
*
      NORM = PDLANGE( '1', NV, NV, C, 1, 1, DESCA, WORK )
*
      QTQNRM2 = NORM / ( DBLE( MAX( MS, 1 ) )*ULP )
*
      IF( QTQNRM2.GT.THRESH )
     $   RES = 1
      RETURN
      END
