*
*==================================================================
*     PB-BLAS LEVEL 2 FUNCTION TEST
*==================================================================
*
      PROGRAM PBZ2CHECK
*
*  -- PB-BLAS Level 2 Double Complex precision Checker routine (version 2.0) --
*     Jaeyoung Choi, Oak Ridge National Laboratory
*     Jack Dongarra, University of Tennessee and Oak Ridge National Lab.
*     David Walker,  Oak Ridge National Laboratory
*     February 28, 1995
*
*
*     The user should modify TOTMEM to indicate the maximum amount of memory
*     in bytes his system has.  Remember to leave room in memory for operating
*     system, the BLACS buffer, etc.  INTSIZ and DBLSIZ indicate the length in
*     bytes on the given platform for an integer and a double precision real.
*     For example, on our system with 8 MB of memory, TOTMEM=6500000
*     (leaves 1.5 MB for OS, code, BLACS buffer, etc), the length of a
*     DOUBLE is 8, and an integer takes up 4 bytes.  Some playing around
*     to discover what the maximum value you can set MEMSIZ to may be required.
*     All arrays used by factorization and solve are allocated out of big
*     array called MEM.
*
      INTEGER          NTESTS, TOTMEM, MEMSIZ, ITYSIZ
      COMPLEX*16       ZERO, ONE
      PARAMETER        ( NTESTS = 10 )
      PARAMETER        ( TOTMEM = 2000000 )
      PARAMETER        ( ITYSIZ = 16 )
      PARAMETER        ( ZERO = ( 0.0D+0, 0.0D+0 ) )
      PARAMETER        ( ONE  = ( 1.0D+0, 0.0D+0 ) )
      PARAMETER        ( MEMSIZ = TOTMEM / ITYSIZ )
*     ..
*     .. Local Scalars  ..
      LOGICAL          PBALL, PBERR
      INTEGER          ICONTXT, NOUT, M, N, MB, NB, MZ, NZ
      INTEGER          IAM, NNODES, IPA, IPC, IPX, IPY, IPZ, IPW
      INTEGER          NPROW, NPCOL, MYROW, MYCOL
      INTEGER          NMAT, NBMAT, NGRIDS, MULLEN, II, JJ, KK, INTEST
      INTEGER          IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL
      INTEGER          IASEED, IXSEED, IYSEED, INCX, INCY
      INTEGER          LCA, LCX, LCY, LRA, LRX, LRY
      INTEGER          MM, NN, IX, IY, IZ, JX, JZ
      DOUBLE PRECISION ERROR, TOL, TOLL, XMAX, TALPHA, TZERO
      COMPLEX*16       ALPHA, BETA
      CHARACTER*1      XWORK, YWORK, XCOMM, YCOMM, XYCOMM
      CHARACTER*1      DIAG, BR1ST
      CHARACTER*80     OUTFILE
*     ..
*     .. Arrays ..
      INTEGER          MVAL( NTESTS ), MBVAL( NTESTS ), MZVAL( NTESTS ),
     $                 NBVAL( NTESTS ), NPVAL( NTESTS ),
     $                 NQVAL( NTESTS ), NVAL( NTESTS ), NZVAL(NTESTS)
      COMPLEX*16       MEM(MEMSIZ)
*     ..
*     .. External Functions ..
      LOGICAL          LSAME
      INTEGER          IZAMAX, NUMROC
      DOUBLE PRECISION DLAMCH
      EXTERNAL         LSAME, IZAMAX, NUMROC, DLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC        MAX, DBLE
*     ..
*     .. Executable Statements ..
*
*     Get starting information
*
      CALL BLACS_PINFO( IAM, NNODES )
      ICONTXT = 100
*
      CALL PBZ2CHKINFO( OUTFILE, NOUT, NMAT, MVAL, NVAL, NBMAT, MBVAL,
     $                  NBVAL, MZVAL, NZVAL, NGRIDS, NPVAL, NQVAL,
     $                  ALPHA, BETA, XCOMM, YCOMM, XWORK, YWORK, DIAG,
     $                  BR1ST, MULLEN, IAROW, IACOL, IXROW, IXCOL,
     $                  IYROW, IYCOL, INCX, INCY, IASEED, IXSEED,
     $                  IYSEED, MEM, IAM, NNODES )
*
*     Test only for INCX > 0 & INCY > 0.
*
      IF(      INCX.LT.0 ) THEN
        IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'INCX'
        GO TO 40
      ELSE IF( INCY.LT.0 ) THEN
        IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'INCY'
        GO TO 40
      END IF
*
*     Generate dirty data (not used)
*
      CALL PZMATGEN( ICONTXT, 'No', 'No', 300, 300, 10, 10, MEM(1), 300,
     $               0, 0, IASEED, 0, 300, 0, 300, 0, 0, 1, 1 )
*
      INTEST = 0
      PBALL = .FALSE.
      PBERR = .FALSE.
      TALPHA = DBLE( ALPHA )
      TZERO  = DBLE( ZERO )
*
*     Loop over different processor grids
*
      DO 30 II = 1, NGRIDS
*
        NPROW = NPVAL(II)
        NPCOL = NQVAL(II)
*
        IF( NPROW.LT.1                        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW'
          GO TO 30
        ELSE IF( NPCOL.LT.1                   ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPCOL'
          GO TO 30
        ELSE IF( NPROW*NPCOL.GT.NNODES        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW*NPCOL'
          GO TO 30
        ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IAROW'
          GO TO 30
        ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IACOL'
          GO TO 30
        ELSE IF( IXROW.LT.0 .OR. IXROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IXROW'
          GO TO 30
        ELSE IF( IXCOL.LT.0 .OR. IXCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IXCOL'
          GO TO 30
        ELSE IF( IYROW.LT.0 .OR. IYROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IYROW'
          GO TO 30
        ELSE IF( IYCOL.LT.0 .OR. IYCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IYCOL'
          GO TO 30
        END IF
*
*       Define processor grid
*
        CALL BLACS_GET( -1, 0, ICONTXT )
        CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', NPROW, NPCOL )
        CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*       Go to bottom of processor grid loop if this case doesn't use my node
*
        IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 30
*
        DO 20 JJ = 1, NMAT
*
          M = MVAL(JJ)
          N = NVAL(JJ)
*
          IF( M.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'M'
            GO TO 20
          ELSE IF( N.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'N'
            GO TO 20
          END IF
*
          DO 10 KK = 1, NBMAT
*
            MB = MBVAL(KK)
            NB = NBVAL(KK)
            MZ = MZVAL(KK)
            NZ = NZVAL(KK)
*
            IF( MB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MB'
              GO TO 10
            ELSE IF( NB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NB'
              GO TO 10
            ELSE IF( MZ.LT.0 .OR. MZ.GE.MB ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MZ'
              GO TO 10
            ELSE IF( NZ.LT.0 .OR. NZ.GE.NB ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NZ'
              GO TO 10
            END IF
*
            IF( IAM.EQ.0 ) THEN
              WRITE(NOUT,2000)
              WRITE(NOUT,3000) NPROW, NPCOL, M, N, MB, NB, MZ, NZ
            END IF
*
*==================================================================
*           PBZTRNV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            NN  = N + NZ
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZTRNV ( CASE 1 )
*
            IZ = 0
            IF( MYROW.EQ.IXROW ) IZ = NZ * INCX
            JZ = 0
            IF( MYCOL.EQ.IYCOL ) JZ = NZ * INCY
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, ZERO, MEM(IPY+JZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPX+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 1, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 2 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 2, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 3 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, 0, IYCOL, IYSEED, 0, 1, 0, LCY, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1, -1,
     $                    IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 3, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 4 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'T', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 4, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 5 )
*
            JZ = 0
            IF( MYCOL.EQ.IXCOL ) JZ = NZ * INCX
            IZ = 0
            IF( MYROW.EQ.IYROW ) IZ = NZ * INCY
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, ZERO, MEM(IPY+IZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPX+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 5, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 6 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 6, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 7 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, 0, IYSEED, 0, LRY, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 7, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 8 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1,
     $                     IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'T', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 8, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 9 )
*
            IZ = 0
            IF( MYROW.EQ.IXROW ) IZ = NZ * INCX
            JZ = 0
            IF( MYCOL.EQ.IYCOL ) JZ = NZ * INCY
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'C', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 9, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 10 )
*
            IZ = 0
            IF( MYROW.EQ.IXROW ) IZ = NZ * INCX
            JZ = 0
            IF( MYCOL.EQ.IYCOL ) JZ = NZ * INCY
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, 0, IYCOL, IYSEED, 0, 1, 0, LCY, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'C', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1, -1,
     $                    IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 10, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 11 )
*
            IPY = 1
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, 0, IXSEED, 0, LRX, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRNV( ICONTXT, 'Col', 'C', N, NB, NZ, MEM(IPX+IZ),
     $                    INCX, BETA, MEM(IPY+JZ), INCY, IXROW, -1,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, 1, 0, NN, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 11, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 12 )
*
            JZ = 0
            IF( MYCOL.EQ.IXCOL ) JZ = NZ * INCX
            IZ = 0
            IF( MYROW.EQ.IYROW ) IZ = NZ * INCY
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'C', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, IXROW, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1,
     $                     0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 12, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 13 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, 0, IYSEED, 0, LRY, 0, 1, MYROW, 0,
     $                     NPROW, 1 )
            CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'C', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 13, IAM, PBERR )
*
*           Test PBZTRNV ( CASE 14 )
*
            IPY = 1
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, 0, IXCOL, IXSEED, 0, 1, 0, LCX, 0,
     $                     MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZTRNV( ICONTXT, 'Row', 'C', N, NB, NZ, MEM(IPX+JZ),
     $                    INCX, BETA, MEM(IPY+IZ), INCY, -1, IXCOL,
     $                    IYROW, IYCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPZ = NN + IPX
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', NN, 1, NB, 1,
     $                     MEM(IPX), 1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PBZVECADD( ICONTXT, 'Ge', N, ONE, MEM(IPX+NZ), 1, BETA,
     $                      MEM(IPZ+NZ), 1 )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRNV ', 14, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZTRNV ', 14, IAM, PBERR )
*
*==================================================================
*           PBZGEMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
            MM  = M + MZ
            NN  = N + NZ
            LRA = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = MZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ * LRA
*
*  ======   Results for the sequential computation (Test 1 - Test 4)
*
            IPZ = 1
            IPX = MM + IPZ
            IPA = NN + IPX
            IPW = MM * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL ZGEMV( 'No', M, N, ALPHA, MEM(IPA+MM*NZ+MZ), MAX(1,MM),
     $                  MEM(IPX+NZ), 1, BETA, MEM(IPZ+MZ), 1 )
*
*           Test PBZGEMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IXROW ) IX = NZ * INCX
            IY = 0
            IF( MYROW.EQ.IAROW ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'No', 'Col', 'Col', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IAROW, IYCOL,
     $                    ' ', ' ', YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 1, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 2 )
*
            LRX = NUMROC( NN, NB, MYROW, IXROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( MM, MB, MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IXROW ) IX = NZ * INCX
            IY = 0
            IF( MYCOL.EQ.IYCOL ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'No', 'Col', 'Row', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL,
     $                    ' ', ' ', ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 2, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IACOL ) IX = NZ * INCX
            IY = 0
            IF( MYROW.EQ.IAROW ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'No', 'Row', 'Col', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IACOL, IAROW, IYCOL,
     $                    XCOMM, XWORK, YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 3, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( MM, MB, MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IACOL ) IX = NZ * INCX
            IY = 0
            IF( MYCOL.EQ.IYCOL ) IY = MZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPY),
     $                     INCY, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'No', 'Row', 'Row', M, N, MB, NB, MZ,
     $                    NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IACOL, IYROW, IYCOL,
     $                    ' ', XWORK, ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', M, MB, MZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+MZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 4, IAM, PBERR )
*
*  ======   Results for the sequential computation (Test 5 - Test 8)
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = MM + IPX
            IPW = MM * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZGEMV( 'Trans', M, N, ALPHA, MEM(IPA+MM*NZ+MZ),
     $                  MAX(1,MM), MEM(IPX+MZ), 1, BETA, MEM(IPZ+NZ),
     $                  1 )
*
*           Test PBZGEMV ( CASE 5 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'Trans', 'Col', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IYCOL,
     $                    ' ', XWORK, ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 5, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 6 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'Trans', 'Col', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IACOL,
     $                    XCOMM, XWORK, YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 6, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 7 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'Trans', 'Row', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL,
     $                    ' ', ' ', ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 7, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'Trans', 'Row', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IACOL,
     $                    ' ', ' ', YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 8, IAM, PBERR )
*
*  ======   Results for the sequential computation (Test 9 - Test 12)
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = MM + IPX
            IPW = MM * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZGEMV( 'Conju', M, N, ALPHA, MEM(IPA+MM*NZ+MZ),
     $                  MAX(1,MM), MEM(IPX+MZ), 1, BETA, MEM(IPZ+NZ),
     $                  1 )
*
*           Test PBZGEMV ( CASE 9 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'Conju', 'Col', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IYCOL,
     $                    ' ', XWORK, ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 9, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 10 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1,
     $                     MEM(IPX), 1, IAROW, IXCOL, IXSEED, 0, LRX, 0,
     $                     LCX, MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'Conju', 'Col', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IAROW, IXCOL, IYROW, IACOL,
     $                    XCOMM, XWORK, YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 10, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 11 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGEMV( ICONTXT, 'Conju', 'Row', 'Col', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL,
     $                    ' ', ' ', ' ', MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 11, IAM, PBERR )
*
*           Test PBZGEMV ( CASE 12 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMV( ICONTXT, 'Conju', 'Row', 'Row', M, N, MB, NB,
     $                    MZ, NZ, ALPHA, MEM(IPA+JZ+IZ), MAX(1,LRA),
     $                    MEM(IPX+IX), INCX, BETA, MEM(IPY+IY), INCY,
     $                    IAROW, IACOL, IXROW, IXCOL, IYROW, IACOL,
     $                    ' ', ' ', YWORK, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+IY), INCY,
     $                     MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMV ', 12, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZGEMV ', 12, IAM, PBERR )
*
*==================================================================
*           PBZGERU TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            MM  = M + MZ
            NN  = N + NZ
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
*
            LRA = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = MZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ * LRA
*
*  ======   Results for the sequential computation
*
            IPC = 1
            IPX = MM * NN + IPC
            IPY = MM + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPC), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZGERU( M, N, ALPHA, MEM(IPX+MZ), 1, MEM(IPY+NZ), 1,
     $                  MEM(IPC+MM*NZ+MZ), MAX(1,MM) )
*
*           Test PBZGERU ( CASE 1 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGERU( ICONTXT, 'Col', 'Col', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                    IYROW, IYCOL, IAROW, IACOL, BR1ST, XCOMM,
     $                    XWORK, ' ', ' ', MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERU ', 1, IAM, PBERR )
*
*           Test PBZGERU ( CASE 2 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGERU( ICONTXT, 'Col', 'Row', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                    IYROW, IACOL, IAROW, IACOL, BR1ST, XCOMM,
     $                    XWORK, YCOMM, YWORK, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERU ', 2, IAM, PBERR )
*
*           Test PBZGERU ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGERU( ICONTXT, 'Row', 'Col', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                    IYROW, IYCOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                    ' ', ' ', MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERU ', 3, IAM, PBERR )
*
*           Test PBZGERU ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGERU( ICONTXT, 'Row', 'Row', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                    IYROW, IACOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                    YCOMM, YWORK, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERU ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZGERU ', 4, IAM, PBERR )
*
*==================================================================
*           PBZGERC TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            MM  = M + MZ
            NN  = N + NZ
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
*
            LRA = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = MZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ * LRA
*
*  ======   Results for the sequential computation
*
            IPC = 1
            IPX = MM * NN + IPC
            IPY = MM + IPX
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPC), MM, 0, 0, IASEED, 0, MM, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, MM, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZGERC( M, N, ALPHA, MEM(IPX+MZ), 1, MEM(IPY+NZ), 1,
     $                  MEM(IPC+MM*NZ+MZ), MAX(1,MM) )
*
*           Test PBZGERC ( CASE 1 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGERC( ICONTXT, 'Col', 'Col', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                    IYROW, IYCOL, IAROW, IACOL, BR1ST, XCOMM,
     $                    XWORK, ' ', ' ', MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERC ', 1, IAM, PBERR )
*
*           Test PBZGERC ( CASE 2 )
*
            LRX = NUMROC( MM, MB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYROW.EQ.IAROW ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, 1, MB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGERC( ICONTXT, 'Col', 'Row', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IAROW, IXCOL,
     $                    IYROW, IACOL, IAROW, IACOL, BR1ST, XCOMM,
     $                    XWORK, YCOMM, YWORK, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERC ', 2, IAM, PBERR )
*
*           Test PBZGERC ( CASE 3 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IYROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYROW.EQ.IYROW ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IYROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZGERC( ICONTXT, 'Row', 'Col', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                    IYROW, IYCOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                    ' ', ' ', MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERC ', 3, IAM, PBERR )
*
*           Test PBZGERC ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( MM, MB, MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IX = 0
            IF( MYCOL.EQ.IXCOL ) IX = MZ * INCX
            IY = 0
            IF( MYCOL.EQ.IACOL ) IY = NZ * INCY
*
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IPA = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPA = MAX(0,(LCY-1)*INCY+1) + IPY
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', MM, NN, MB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, MM, 1, MB, MEM(IPX),
     $                     INCX, IXROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGERC( ICONTXT, 'Row', 'Row', M, N, MB, NB, MZ, NZ,
     $                    ALPHA, MEM(IPX+IX), INCX, MEM(IPY+IY), INCY,
     $                    MEM(IPA+JZ+IZ), MAX(1,LRA), IXROW, IXCOL,
     $                    IYROW, IACOL, IAROW, IACOL, BR1ST, ' ', ' ',
     $                    YCOMM, YWORK, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, M, N, MB, NB, MZ, NZ,
     $                     MEM(IPA+JZ+IZ), LRA, MEM(IPC+MM*NZ+MZ), MM,
     $                     IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGERC ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZGERC ', 4, IAM, PBERR )
*
*==================================================================
*           PBZHEMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')

            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (TEST 1 - TEST 2)
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = NN + IPX
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHEMV( 'Upper', N, ALPHA, MEM(IPA+NN*NZ+NZ), MAX(1,NN),
     $                  MEM(IPX+NZ), 1, BETA, MEM(IPZ+NZ), 1 )
*
*           Test PBZHEMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA,
     $                     IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZHEMV( ICONTXT, 'Upper', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, BETA,
     $                    MEM(IPY+IZ*INCY), INCY, IAROW, IACOL, IXCOL,
     $                    IYCOL, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMV ', 1, IAM, PBERR )
*
*           Test PBZHEMV ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMV( ICONTXT, 'Upper', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, BETA,
     $                    MEM(IPY+JZ*INCY), INCY, IAROW, IACOL, IXROW,
     $                    IYROW, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMV ', 2, IAM, PBERR )
*
*           Results for the sequential computation (TEST 3 - TEST 4)
*
            IPZ = 1
            IPX = NN + IPZ
            IPA = NN + IPX
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHEMV( 'Lower', N, ALPHA, MEM(IPA+NN*NZ+NZ), MAX(1,NN),
     $                  MEM(IPX+NZ), 1, BETA, MEM(IPZ+NZ), 1 )
*
*           Test PBZHEMV ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LRY-1)*INCY+1) + IPY
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZHEMV( ICONTXT, 'Lower', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, BETA,
     $                    MEM(IPY+IZ*INCY), INCY, IAROW, IACOL, IXCOL,
     $                    IYCOL, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPY+IZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IAROW, IYCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMV ', 3, IAM, PBERR )
*
*           Test PBZHEMV ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPY = IPX
            IPX = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPX = MAX(0,(LCY-1)*INCY+1) + IPY
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMV( ICONTXT, 'Lower', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, BETA,
     $                    MEM(IPY+JZ*INCY), INCY, IAROW, IACOL, IXROW,
     $                    IYROW, XWORK, YWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPY+JZ*INCY),
     $                     INCY, MEM(IPZ+NZ), 1, IYROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMV ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZHEMV ', 4, IAM, PBERR )
*
*==================================================================
*           PBZHER TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (Test 1 - Test 2)
*
            IPC = 1
            IPX = NN * NN + IPC
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHER( 'Upper', N, TALPHA, MEM(IPX+NZ), 1,
     $                 MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBZHER ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZHER( ICONTXT, 'Upper', 'Col', N, NB, NZ, TALPHA,
     $                   MEM(IPX+IZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXCOL, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER  ', 1, IAM, PBERR )
*
*           Test PBZHER ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER( ICONTXT, 'Upper', 'Row', N, NB, NZ, TALPHA,
     $                   MEM(IPX+JZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXROW, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER  ', 2, IAM, PBERR )
*
*           Results for the sequential computation (Test 3 - Test 4)
*
            IPC = 1
            IPX = NN * NN + IPC
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHER( 'Lower', N, TALPHA, MEM(IPX+NZ), 1,
     $                 MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBZHER ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LRX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZHER( ICONTXT, 'Lower', 'Col', N, NB, NZ, TALPHA,
     $                   MEM(IPX+IZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXCOL, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER  ', 3, IAM, PBERR )
*
*           Test PBZHER ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPW = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPW = MAX(0,(LCX-1)*INCX+1) + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER( ICONTXT, 'Lower', 'Row', N, NB, NZ, TALPHA,
     $                   MEM(IPX+JZ*INCX), INCX, MEM(IPA+JZ*LRA+IZ),
     $                   MAX(1,LRA), IXROW, IAROW, IACOL, XCOMM, XWORK,
     $                   'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER  ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZHER  ', 4, IAM, PBERR )
*
*==================================================================
*           PBZHER2 TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            XYCOMM = XCOMM
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (Test 1 - Test 2)
*
            IPC = 1
            IPX = NN * NN + IPC
            IPY = NN + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHER2( 'Upper', N, ALPHA, MEM(IPX+NZ), 1, MEM(IPY+NZ),
     $                  1, MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBZHER2 ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LRY-1)*INCY+1) + IPY
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZHER2( ICONTXT, 'Upper', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPX+IZ*INCX), INCX, MEM(IPY+IZ*INCY),
     $                    INCX, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXCOL,
     $                    IYCOL, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2 ', 1, IAM, PBERR )
*
*           Test PBZHER2 ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LCY-1)*INCY+1) + IPY
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2( ICONTXT, 'Upper', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPX+JZ*INCX), INCX, MEM(IPY+JZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXROW,
     $                    IYROW, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2 ', 2, IAM, PBERR )
*
*           Results for the sequential computation (Test 3 - Test 4)
*
            IPC = 1
            IPX = NN * NN + IPC
            IPY = NN + IPX
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPC), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, 0, 0, IYSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZHER2( 'Lower', N, ALPHA, MEM(IPX+NZ), 1, MEM(IPY+NZ),
     $                  1, MEM(IPC+NN*NZ+NZ), MAX(1,NN) )
*
*           Test PBZHER2 ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
            LRY = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCY = NUMROC( 1,  1,  MYCOL, IYCOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = MAX(0,(LRY-1)*INCY+1) * LCY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LRY-1)*INCY+1) + IPY
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPY),
     $                     1, IAROW, IYCOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IYCOL )
     $        CALL ZVREDIST( LRY, ONE, MEM(IPY), 1, INCY )
            CALL PBZHER2( ICONTXT, 'Lower', 'Col', N, NB, NZ, ALPHA,
     $                    MEM(IPX+IZ*INCX), INCX, MEM(IPY+IZ*INCY),
     $                    INCX, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXCOL,
     $                    IYCOL, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2 ', 3, IAM, PBERR )
*
*           Test PBZHER2 ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
            LRY = NUMROC( 1,  1,  MYROW, IYROW, NPROW )
            LCY = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPA = IPX
            IPX = LRA * LCA + IPA
            IPY = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPY = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = MAX(0,(LCY-1)*INCY+1) * LRY + IPY
            IF( LSAME( YWORK, 'Y' ) ) IPW = MAX(0,(LCY-1)*INCY+1) + IPY
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPY),
     $                     INCY, IYROW, IACOL, IYSEED, 0, LRY, 0, LCY,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2( ICONTXT, 'Lower', 'Row', N, NB, NZ, ALPHA,
     $                    MEM(IPX+JZ*INCX), INCX, MEM(IPY+JZ*INCY),
     $                    INCY, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA), IXROW,
     $                    IYROW, IAROW, IACOL, XYCOMM, XWORK, YWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
            CALL PZMT2CHK( ICONTXT, N, N, NB, NB, NZ, NZ,
     $                     MEM(IPA+JZ*LRA+IZ), LRA, MEM(IPC+NN*NZ+NZ),
     $                     NN, IAROW, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2 ', 4, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZHER2 ', 4, IAM, PBERR )
*
*==================================================================
*           PBZTRMV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Results for the sequential computation (TEST 1 - TEST 2)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Upper', 'No', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 1, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 2 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 2, IAM, PBERR )
*
*           Results for the sequential computation (TEST 3 - TEST 4)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Lower', 'No', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 3, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 4 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA,  MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 4, IAM, PBERR )
*
*           Results for the sequential computation (TEST 5 - TEST 6)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Upper', 'Trans', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 5 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 5, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 6 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 6, IAM, PBERR )
*
*           Results for the sequential computation (TEST 7 - TEST 8)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Lower', 'Trans', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 7 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 7, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 8, IAM, PBERR )
*
*           Results for the sequential computation (TEST 9 - TEST 10)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Upper', 'Conju', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 9 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Upper', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 9, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 10 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Upper', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 10, IAM, PBERR )
*
*           Results for the sequential computation (TEST 11 - TEST 12)
*
            IPZ = 1
            IPA = NN + IPZ
            IPW = NN * NN + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), NN, 0, 0, IASEED, 0, NN, 0, NN, 0,
     $                     0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL ZTRMV( 'Lower', 'Conju', DIAG, N, MEM(IPA+NN*NZ+NZ),
     $                  MAX(1,NN), MEM(IPZ+NZ), 1 )
*
*           Test PBZTRMV ( CASE 11 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA,
     $                     IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRMV( ICONTXT, 'Lower', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 11, IAM, PBERR )
*
*           Test PBZTRMV ( CASE 12 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = IPA
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMV( ICONTXT, 'Lower', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MULLEN, MEM(IPW) )
*
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMV ', 12, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZTRMV ', 12, IAM, PBERR )
*
*==================================================================
*           PBZTRSV TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            NN  = N + NZ
            LRA = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IZ = 0
            IF( MYROW.EQ.IAROW ) IZ = NZ
            JZ = 0
            IF( MYCOL.EQ.IACOL ) JZ = NZ
*
*           Test PBZTRSV ( CASE 1 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 1, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 2 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 2, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 3 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 3, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 4 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 4, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 5 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 5, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 6 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'No', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1,  0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 6, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 7 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 7, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 8 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'Trans', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 8, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 9 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Lower', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 9, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 10 )
*
            LRX = NUMROC( NN, NB, MYROW, IAROW, NPROW )
            LCX = NUMROC( 1,  1,  MYCOL, IXCOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LRX-1)*INCX+1) * LCX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LRX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPX),
     $                     1, IAROW, IXCOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            IF( MYCOL.EQ.IXCOL )
     $        CALL ZVREDIST( LRX, ONE, MEM(IPX), 1, INCX )
            CALL PBZTRSV( ICONTXT, 'Upper', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    ' ', MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LRX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'Conju', DIAG, 'Col', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+IZ*INCX), INCX, IAROW, IACOL, IXCOL,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Col', N, NB, NZ, MEM(IPX+IZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IAROW, IXCOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 10, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 11 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Lower', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Lower', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 11, IAM, PBERR )
*
*           Test PBZTRSV ( CASE 12 )
*
            LRX = NUMROC( 1,  1,  MYROW, IXROW, NPROW )
            LCX = NUMROC( NN, NB, MYCOL, IACOL, NPCOL )
*
            IPX = 1
            IPA = MAX(0,(LCX-1)*INCX+1) * LRX + IPX
            IF( LSAME( XWORK, 'Y' ) ) IPA = MAX(0,(LCX-1)*INCX+1) + IPX
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, NN, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', 1, NN, 1, NB, MEM(IPX),
     $                     INCX, IXROW, IACOL, IXSEED, 0, LRX, 0, LCX,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSV( ICONTXT, 'Upper', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    XWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPX)
*
            IF( LRX*LCX.GT.0 ) THEN
              XMAX = ABS( MEM(IPX+(IZAMAX(LCX,MEM(IPX),INCX)-1)*INCX) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMV
*
            CALL PBZTRMV( ICONTXT, 'Upper', 'Conju', DIAG, 'Row', N, NB,
     $                    NZ, MEM(IPA+JZ*LRA+IZ), MAX(1,LRA),
     $                    MEM(IPX+JZ*INCX), INCX, IAROW, IACOL, IXROW,
     $                    'No', MULLEN, MEM(IPW) )
*
            IPZ = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', NN, 1, NB, 1, MEM(IPZ),
     $                     1, 0, 0, IXSEED, 0, NN, 0, 1, 0, 0, 1, 1 )
            CALL PZVECCHK( ICONTXT, 'Row', N, NB, NZ, MEM(IPX+JZ*INCX),
     $                     INCX, MEM(IPZ+NZ), 1, IXROW, IACOL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSV ', 12, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZTRSV ', 12, IAM, PBERR )
*
            INTEST = INTEST + 1
            IF( IAM.EQ.0 ) WRITE( NOUT, 4000 ) INTEST
   10     CONTINUE
   20   CONTINUE
        CALL BLACS_GRIDEXIT( ICONTXT )
   30 CONTINUE
   40 CONTINUE
*
      IF( IAM.EQ.0 ) THEN
         IF( PBALL ) THEN
            WRITE( NOUT, 5000 )
         ELSE
            WRITE( NOUT, 6000 ) INTEST
         END IF
         WRITE( NOUT, 7000 )
      END IF
*
      CALL BLACS_EXIT( 0 )
*
 1000 FORMAT( /, 'Illegal input value of ', A11 )
 2000 FORMAT(/, '    P    Q     M    N    MB  NB  MZ  NZ')
 3000 FORMAT(1X,I4,1X,I4,3X,I4,1X,I4,2X,I3,1X,I3,1X,I3,1X,I3)
 4000 FORMAT( '-- PBZ2CHECK TEST SET (', I4, ' ) IS COMPLETED --')
 5000 FORMAT( /, '== THERE WERE FAILURES IN THIS RUN ==')
 6000 FORMAT( /, '== THERE WERE NO FAILURES IN (', I4, ' ) ',
     $        'TEST SETS ==')
 7000 FORMAT( /, 'End of PBZ2CHECK TESTS.' )
*
      STOP
*
*     End of PBZ2CHECK
*
      END
*
*==================================================================
*     SUBROUTINE ZVREDIST  -- change INCX1 to INCX2
*==================================================================
*
      SUBROUTINE ZVREDIST( N, BETA, X, INCX1, INCX2 )
*
*  -- PB-BLAS Auxiliary Routine (version 2.0) --
*     Oak Ridge National Laboratory and University of Tennessee.
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      INTEGER              N, INCX1, INCX2
      COMPLEX*16           BETA
*     ..
*     .. Array Arguments ..
      COMPLEX*16           X( * )
*     ..
*     .. Exteranl Subroutine ..
      EXTERNAL             ZSCAL
*     ..
*     .. Parameters ..
      COMPLEX*16           ZERO
      PARAMETER            ( ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      INTEGER              I, IX1, IX2
*     ..
*
      IF( BETA.NE.ZERO ) THEN
        IF( INCX1.LT.INCX2 ) THEN
          IX1 = (N-1) * INCX1 + 1
          IX2 = (N-1) * INCX2 + 1
*
          DO 10 I = 1, N
            X( IX2 ) = BETA * X( IX1 )
            IX1 = IX1 - INCX1
            IX2 = IX2 - INCX2
   10     CONTINUE
*
        ELSE IF( INCX1.GT.INCX2 ) THEN
          IX1 = 1
          IX2 = 1
*
          DO 20 I = 1, N
            X( IX2 ) = BETA * X( IX1 )
            IX1 = IX1 + INCX1
            IX2 = IX2 + INCX2
   20     CONTINUE
*
        ELSE
          CALL ZSCAL( N, BETA, X, INCX1 )
        END IF
*
      ELSE
        IX2 = 1
        DO 30 I = 1, N
            X( IX2 ) = ZERO
            IX2 = IX2 + INCX2
   30   CONTINUE
      END IF
*
      RETURN
*
*     End of ZVREDIST
*
      END
