      PROGRAM PBZ3CHECK
*
*  -- PB-BLAS Level 3 Double Complex precision Checker routine (version 2.0) --
*     Jaeyoung Choi, Oak Ridge National Laboratory
*     Jack Dongarra, University of Tennessee and Oak Ridge National Lab.
*     David Walker,  Oak Ridge National Laboratory
*     February 28, 1995
*
*
*     The user should modify TOTMEM to indicate the maximum amount of memory
*     in bytes his system has.  Remember to leave room in memory for operating
*     system, the BLACS buffer, etc.  INTSIZ and DBLSIZ indicate the length in
*     bytes on the given platform for an integer and a double precision real.
*     For example, on our system with 8 MB of memory, TOTMEM=6500000
*     (leaves 1.5 MB for OS, code, BLACS buffer, etc), the length of a
*     DOUBLE is 8, and an integer takes up 4 bytes.  Some playing around
*     to discover what the maximum value you can set MEMSIZ to may be required.
*     All arrays used by factorization and solve are allocated out of big
*     array called MEM.
*
*     ..
*     .. Parameters ..
      INTEGER          NTESTS, TOTMEM, MEMSIZ, ITYSIZ
      COMPLEX*16       ZERO, ONE
      PARAMETER        ( NTESTS = 10 )
      PARAMETER        ( TOTMEM = 2000000 )
      PARAMETER        ( ITYSIZ = 16 )
      PARAMETER        ( ZERO = ( 0.0D+0, 0.0D+0 ) )
      PARAMETER        ( ONE  = ( 1.0D+0, 0.0D+0 ) )
      PARAMETER        ( MEMSIZ = TOTMEM / ITYSIZ )
*     ..
*     .. Local Scalars  ..
      LOGICAL          PBALL, PBERR
      INTEGER          ICONTXT, NOUT, M, N, K, MB, NB, KB
      INTEGER          IAM, NNODES, IPA, IPB, IPC, IPD, IPW
      INTEGER          NPROW, NPCOL, MYROW, MYCOL, II, IX, JJ, JX, KK
      INTEGER          NMAT, NBMAT, NGRIDS, MULLEN, INTEST
      INTEGER          IAROW, IACOL, IBROW, IBCOL, ICROW, ICCOL
      INTEGER          IASEED, IBSEED, ICSEED
      INTEGER          LCA, LCB, LCC, LRA, LRB, LRC
      INTEGER          IAPOS, IBPOS, ICPOS, IMROW, IMCOL
      DOUBLE PRECISION ERROR, TOL, TOLL, TALPHA, TBETA, TZERO, XMAX
      COMPLEX*16       ALPHA, BETA
      CHARACTER*1      AWORK, BWORK, CWORK, ACOMM, BCOMM, CCOMM, ABCOMM
      CHARACTER*1      DIAG, BR1ST
      CHARACTER*80     OUTFILE
*     ..
*     .. Arrays ..
      INTEGER          KBVAL( NTESTS ), KVAL( NTESTS ), MVAL( NTESTS ),
     $                 MBVAL( NTESTS ), NBVAL( NTESTS ),
     $                 NPVAL( NTESTS), NQVAL( NTESTS ), NVAL( NTESTS )
      COMPLEX*16       MEM( MEMSIZ )
*     ..
*     .. External Functions ..
      LOGICAL          LSAME
      INTEGER          IZAMAX, NUMROC
      DOUBLE PRECISION DLAMCH
      EXTERNAL         LSAME, IZAMAX, NUMROC, DLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC        ABS, DBLE, MAX
*     ..
*     .. Executable Statements ..
*
*     Get starting information
*
      CALL BLACS_PINFO( IAM, NNODES )
*
      CALL PBZ3CHKINFO( OUTFILE, NOUT, NMAT, MVAL, NVAL, KVAL, NBMAT,
     $                  MBVAL, NBVAL, KBVAL, NGRIDS, NPVAL, NQVAL,
     $                  ALPHA, BETA, ACOMM, BCOMM, AWORK, BWORK, CWORK,
     $                  DIAG, BR1ST, MULLEN, IAROW, IACOL, IBROW, IBCOL,
     $                  ICROW, ICCOL, IASEED, IBSEED, ICSEED, MEM, IAM,
     $                  NNODES )
*
*     Generate dirty data (not used)
*
      CALL PZMATGEN( ICONTXT, 'No', 'No', 300, 300, 10, 10, MEM(1), 300,
     $               0, 0, IASEED, 0, 300, 0, 300, 0, 0, 1, 1 )
*
      INTEST = 0
      PBALL = .FALSE.
      PBERR = .FALSE.
      TALPHA = DBLE( ALPHA )
      TBETA  = DBLE( BETA )
      TZERO  = DBLE( ZERO )
*
*     Loop over different processor grids
*
      DO 30 II = 1, NGRIDS
*
        NPROW = NPVAL(II)
        NPCOL = NQVAL(II)
*
        IF( NPROW.LT.1                        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW'
          GO TO 30
        ELSE IF( NPCOL.LT.1                   ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPCOL'
          GO TO 30
        ELSE IF( NPROW*NPCOL.GT.NNODES        ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NPROW*NPCOL'
          GO TO 30
        ELSE IF( IAROW.LT.0 .OR. IAROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IAROW'
          GO TO 30
        ELSE IF( IACOL.LT.0 .OR. IACOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IACOL'
          GO TO 30
        ELSE IF( IBROW.LT.0 .OR. IBROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IBROW'
          GO TO 30
        ELSE IF( IBCOL.LT.0 .OR. IBCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'IBCOL'
          GO TO 30
        ELSE IF( ICROW.LT.0 .OR. ICROW.GE.NPROW ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'ICROW'
          GO TO 30
        ELSE IF( ICCOL.LT.0 .OR. ICCOL.GE.NPCOL ) THEN
          IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'ICCOL'
          GO TO 30
        END IF
*
*       Define processor grid
*
        CALL BLACS_GET( -1, 0, ICONTXT )
        CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', NPROW, NPCOL )
        CALL BLACS_GRIDINFO( ICONTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*       Go to bottom of processor grid loop if this case doesn't use my node
*
        IF( MYROW.GE.NPROW .OR. MYCOL.GE.NPCOL ) GO TO 30
*
        DO 20 JJ = 1, NMAT
*
          M = MVAL(JJ)
          N = NVAL(JJ)
          K = KVAL(JJ)
*
          IF( M.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'M'
            GO TO 20
          ELSE IF( N.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'N'
            GO TO 20
          ELSE IF( K.LT.0 ) THEN
            IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'K'
            GO TO 20
          END IF
*
          DO 10 KK = 1, NBMAT
            MB = MBVAL(KK)
            NB = NBVAL(KK)
            KB = KBVAL(KK)
*
            IF( MB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'MB'
              GO TO 10
            ELSE IF( NB.LT.0 ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'NB'
              GO TO 10
            ELSE IF( NB.LT.0 .OR. KB.LT.K ) THEN
              IF( IAM.EQ.0 ) WRITE(NOUT,1000) 'KB'
              GO TO 10
            END IF
*
            IF( IAM.EQ.0 ) THEN
              WRITE(NOUT,2000)
              WRITE(NOUT,3000) NPROW, NPCOL, M, N, K, MB, NB, KB
            END IF
*
*==================================================================
*           PBZTRAN TEST
*==================================================================
*
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
*
*           Test PBZTRAN ( CASE 1 )
*
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), ZERO, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPA),
     $                     MAX(1,K), ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 1, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 2, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 3 )
*
            IPC = 1
            IPA = K * LCC + IPC
            IPW = K * LRA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, 0, ICCOL, ICSEED, 0, K, 0, LCC,
     $                     0, MYCOL, 1, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, -1, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), MAX(1,K), 0, 0, IASEED, 0, K, 0, N,
     $                     0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     MAX(1,K), 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1,
     $                     1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 3, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LRA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'T', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 4, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 5 )
*
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), ZERO, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPA), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 5, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 6 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 6, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 7 )
*
            IPC = 1
            IPA = K * LRC + IPC
            IPW = K * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, 0, ICSEED, 0, LRC, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PBZTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                     BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 7, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 8 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Row', 'T', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Trans', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N,  0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 8, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 9 )
*
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'C', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 9, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 10 )
*
            IPC = 1
            IPA = K * LCC + IPC
            IPW = K * LRA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, 0, ICCOL, ICSEED, 0, K, 0, LCC,
     $                     0, MYCOL, 1, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'C', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, -1, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 10, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 11 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LRA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAROW, 0, IASEED, 0, LRA, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     K, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Col', 'C', N, K, NB, MEM(IPA),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    -1, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', K, N, KB, NB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'G', K, N, ONE, MEM(IPA), MAX(1,K),
     $                      BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 11, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 12 )
*
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Row', 'C', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 12, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 13 )
*
            IPC = 1
            IPA = K * LRC + IPC
            IPW = K * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, 0, ICSEED, 0, LRC, 0, K,
     $                     MYROW, 0, NPROW, 1 )
            CALL PBZTRAN( ICONTXT, 'Row', 'C', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, -1, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = K * N + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', N, K, NB, KB,
     $                      MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                      1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 13, IAM, PBERR )
*
*           Test PBZTRAN ( CASE 14 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = K   * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, IACOL, IASEED, 0, K, 0, LCA,
     $                     0, MYCOL, 1, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRAN( ICONTXT, 'Row', 'C', K, N, NB, MEM(IPA),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), -1,
     $                    IACOL, ICROW, ICCOL, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'Conju', 'No', N, K, NB, KB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PBZMATADD( ICONTXT, 'V', N, K, ONE, MEM(IPA), MAX(1,N),
     $                      BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRAN ', 14, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZTRAN ', 14, IAM, PBERR )
*
*==================================================================
*           PBZGEMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(MAX(M,N)) * DLAMCH('eps')
*
*           Test PBZGEMM ( CASE 1 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'No', 'No', M, N, K, MB, NB, KB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IACOL, IBROW, IMCOL, IMROW, IMCOL, BR1ST,
     $                    ACOMM, AWORK, BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'No', M, N, K, ALPHA, MEM(IPA), MAX(1,M),
     $                  MEM(IPB), MAX(1,K), BETA, MEM(IPD), MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 1, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 2 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = LRC * K + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, IMROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'No', 'No', M, K, N, MB, KB, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IMCOL, IBROW, IBCOL, IMROW, ICCOL, ' ', ' ',
     $                    ' ', ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'No', M, K, N, ALPHA, MEM(IPA), MAX(1,M),
     $                  MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 2, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 3 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'No', 'No', K, N, M, KB, NB, MB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IMROW, IMCOL, ICROW, IMCOL, ' ', ' ',
     $                    ' ', ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     M, 0, 0, IBSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'No', K, N, M, ALPHA, MEM(IPA), MAX(1,K),
     $                  MEM(IPB), MAX(1,M), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 3, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 4 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'Trans', 'No', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBROW, IMCOL, IMROW, IMCOL, BR1ST, ' ',
     $                    ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'No', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 4, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 5 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRB * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IMROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'Trans', 'No', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IMROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'No', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 5, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 6 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPB
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'Trans', 'No', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IMROW,
     $                    IACOL, IMROW, IMCOL, ICROW, IMCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     M, 0, 0, IBSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'No', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,M), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 6, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 7 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'No', 'Trans', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ACOMM, AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Trans', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 7, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 8 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = LRC * K + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, IMROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'No', 'Trans', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IMCOL, IBROW, IMCOL, IMROW, ICCOL, ' ', ' ',
     $                    ' ', BCOMM, BWORK, CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Trans', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 8, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 9 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'No', 'Trans', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IMCOL, IMROW, IMCOL, ICROW, ICCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Trans', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 9, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 10 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'Trans', 'Trans', M, N, K, MB,
     $                    NB, KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'Trans', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 10, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 11 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'Trans', 'Trans', M, K, N, MB,
     $                    KB, NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IBROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'Trans', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 11, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 12 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'Trans', 'Trans', K, N, M, KB,
     $                    NB, MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IMROW, IMCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Trans', 'Trans', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 12, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 13 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'Conju', 'No', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBROW, IMCOL, IMROW, IMCOL, BR1ST, ' ',
     $                    ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'No', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 13, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 14 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRB * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IMROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'Conju', 'No', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IMROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', BCOMM, BWORK, ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'No', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 14, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 15 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPB
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'Conju', 'No', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IMROW,
     $                    IACOL, IMROW, IMCOL, ICROW, IMCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPB),
     $                     M, 0, 0, IBSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'No', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,M), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 15, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 16 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRA * K + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IMROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'No', 'Conju', M, N, K, MB, NB,
     $                    KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ACOMM, AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Conju', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 16, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 17 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = LRC * K + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, IMROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'No', 'Conju', M, K, N, MB, KB,
     $                    NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IMROW,
     $                    IMCOL, IBROW, IMCOL, IMROW, ICCOL, ' ', ' ',
     $                    ' ', BCOMM, BWORK, CWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Conju', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 17, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 18 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'No', 'Conju', K, N, M, KB, NB,
     $                    MB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IMCOL, IMROW, IMCOL, ICROW, ICCOL, ' ', ACOMM,
     $                    AWORK, ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'No', 'Conju', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 18, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 19 )
*
            IMROW = ICROW
            IMCOL = ICCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBROW, NPROW )
            LCB = NUMROC( K, KB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, IMROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, IMCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, KB, MB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPC),
     $                     LRC, IMROW, IMCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'C', 'Conju', 'Conju', M, N, K, MB,
     $                    NB, KB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IBROW, IBCOL, IMROW, IMCOL,
     $                    BR1ST, ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * N + IPD
            IPB = M * K + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, M, MB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, N, MB, NB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'Conju', M, N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, N, MB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), M, IMROW, IMCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 19, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 20 )
*
            IMROW = IAROW
            IMCOL = IACOL
            LRA = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCA = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBROW, NPROW )
            LCB = NUMROC( N, NB, MYCOL, IBCOL, NPCOL )
            LRC = NUMROC( M, MB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     LRA, IMROW, IMCOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBROW, IBCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'A', 'Conju', 'Conju', M, K, N, MB,
     $                    KB, NB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IMROW, IMCOL, IBROW, IBCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = M * K + IPD
            IPB = M * N + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPD),
     $                     M, 0, 0, ICSEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'Conju', M, K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,M) )
*
            CALL PZMATCHK( ICONTXT, M, K, MB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), M, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 20, IAM, PBERR )
*
*           Test PBZGEMM ( CASE 21 )
*
            IMROW = IBROW
            IMCOL = IBCOL
            LRA = NUMROC( M, MB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IMROW, NPROW )
            LCB = NUMROC( M, MB, MYCOL, IMCOL, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCA + IPA
            IPW = LRB * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     LRB, IMROW, IMCOL, IBSEED, 0, LRB, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZGEMM( ICONTXT, 'B', 'Conju', 'Conju', K, N, M, KB,
     $                    NB, MB, ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC),
     $                    IAROW, IACOL, IMROW, IMCOL, ICROW, ICCOL,
     $                    ' ', ' ', ' ', ' ', ' ', ' ', MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPB = K * M + IPA
            IPW = M * N + IPW
            CALL PZMATGEN( ICONTXT, 'No', 'No', M, K, MB, KB, MEM(IPA),
     $                     M, 0, 0, IASEED, 0, M, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, M, NB, MB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, M, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZGEMM( 'Conju', 'Conju', K, N, M, ALPHA, MEM(IPA),
     $                  MAX(1,M), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZGEMM ', 21, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZGEMM ', 21, IAM, PBERR )
*
*==================================================================
*           PBZHEMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZHEMM ( CASE 1 )
*
            IBPOS = IBCOL
            ICPOS = ICCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( K, KB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Matrix', 'Left', 'Lower', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZHEMM( 'Left', 'Lower', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 1, IAM, PBERR )
*
*           Test PBZHEMM ( CASE 2 )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Matrix', 'Left', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZHEMM( 'Left', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 2, IAM, PBERR )
*
*           Test PBZHEMM ( CASE 3 )
*
            IBPOS = IBROW
            ICPOS = ICROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Matrix', 'Right', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZHEMM( 'Right', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 3, IAM, PBERR )
*
*           Test PBZHEMM ( CASE 4 )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Matrix', 'Right', 'Upper', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZHEMM( 'Right', 'Upper', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 4, IAM, PBERR )
*
*           Test PBZHEMM ( CASE 5 )
*
            IBPOS = IBCOL
            ICPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( N, NB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Block', 'Left', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB), MAX(1,K),
     $                    BETA, MEM(IPC), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ICPOS, ACOMM, AWORK, ' ', 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZHEMM( 'Left', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 5, IAM, PBERR )
*
*           Test PBZHEMM ( CASE 6 )
*
            IBPOS = IBROW
            ICPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHEMM( ICONTXT, 'Block', 'Right', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBPOS, ICPOS, ACOMM, AWORK, ' ', 0,
     $                    MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPA), K, 0, 0, IASEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZHEMM( 'Right', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHEMM ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZHEMM ', 6, IAM, PBERR )
*
*==================================================================
*           PBZHERK TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZHERK ( CASE 1 )
*
            IAPOS = IACOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                    TALPHA, MEM(IPA), MAX(1,LRC), TBETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM,
     $                    AWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Lower', 'No', N, K, TALPHA, MEM(IPA), MAX(1,N),
     $                  TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 1, IAM, PBERR )
*
*           Test PBZHERK ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                    TALPHA, MEM(IPA), MAX(1,LRC), TBETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM,
     $                    AWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Upper', 'No', N, K, TALPHA, MEM(IPA), MAX(1,N),
     $                  TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 2, IAM, PBERR )
*
*           Test PBZHERK ( CASE 3 )
*
            IAPOS = IAROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Matrix', 'Lower', 'Conju', N, K, NB,
     $                    TALPHA, MEM(IPA), MAX(1,K), TBETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Lower', 'Conju', N, K, TALPHA, MEM(IPA),
     $                  MAX(1,K), TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 3, IAM, PBERR )
*
*           Test PBZHERK ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Matrix', 'Upper', 'Conju', N, K, NB,
     $                    TALPHA, MEM(IPA), MAX(1,K), TBETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Upper', 'Conju', N, K, TALPHA, MEM(IPA),
     $                  MAX(1,K), TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 4, IAM, PBERR )
*
*           Test PBZHERK ( CASE 5 )
*
            IAPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRC * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                    TALPHA, MEM(IPA), MAX(1,K), TBETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPD), K, 0, 0, ICSEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Upper', 'No', K, N, TALPHA, MEM(IPA), MAX(1,K),
     $                  TBETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 5, IAM, PBERR )
*
*           Test PBZHERK ( CASE 6 )
*
            IAPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRA * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHERK( ICONTXT, 'Block', 'Lower', 'Conju', K, N, NB,
     $                    TALPHA, MEM(IPA), MAX(1,LRA), TBETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPD), K, 0, 0, ICSEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL ZHERK( 'Lower',  'Conju', K, N, TALPHA, MEM(IPA),
     $                  MAX(1,N), TBETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHERK ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZHERK ', 6, IAM, PBERR )
*
*==================================================================
*           PBZHER2K TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            ABCOMM = ACOMM
*
*           Test PBZHER2K ( CASE 1 )
*
            IAPOS  = IACOL
            IBPOS  = IBCOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), TBETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Lower', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 1, IAM, PBERR )
*
*           Test PBZHER2K ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), TBETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Upper', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), TBETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 2, IAM, PBERR )
*
*           Test PBZHER2K ( CASE 3 )
*
            IAPOS = IAROW
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Matrix', 'Lower', 'Conju', N, K,
     $                     NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), TBETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Lower', 'Conju', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), TBETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 3, IAM, PBERR )
*
*           Test PBZHER2K ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Matrix', 'Upper', 'Conju', N, K,
     $                     NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), TBETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', N, N, NB, NB,
     $                     MEM(IPD), N, 0, 0, ICSEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Upper', 'Conju', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), TBETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 4, IAM, PBERR )
*
*           Test PBZHER2K ( CASE 5 )
*
            IAPOS = IACOL
            IBPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRC * LCA + IPA
            IPW = LRC * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), TBETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPD), K, 0, 0, ICSEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Upper', 'No', K, N, ALPHA, MEM(IPA), MAX(1,K),
     $                   MEM(IPB), MAX(1,K), TBETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 5, IAM, PBERR )
*
*           Test PBZHER2K ( CASE 6 )
*
            IAPOS = IAROW
            IBPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRA * LCC + IPA
            IPW = LRB * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPC), LRC, ICROW, ICCOL, ICSEED, 0, LRC,
     $                     0, LCC, MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZHER2K( ICONTXT, 'Block', 'Lower', 'Conju', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                     MAX(1,LRB), TBETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'Hemm', 'No', K, K, KB, KB,
     $                     MEM(IPD), K, 0, 0, ICSEED, 0, K, 0, K, 0, 0,
     $                     1, 1 )
            CALL ZHER2K( 'Lower',  'Conju', K, N, ALPHA, MEM(IPA),
     $                   MAX(1,N), MEM(IPB), MAX(1,N), TBETA, MEM(IPD),
     $                   MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZHER2K ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZHER2K ', 6, IAM, PBERR )
*
*==================================================================
*           PBZSYMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZSYMM ( CASE 1 )
*
            IBPOS = IBCOL
            ICPOS = ICCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( K, KB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Matrix', 'Left', 'Lower', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZSYMM( 'Left', 'Lower', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 1, IAM, PBERR )
*
*           Test PBZSYMM ( CASE 2 )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = LRA * K + IPC
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Matrix', 'Left', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,LRA), BETA, MEM(IPC), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZSYMM( 'Left', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRA,
     $                     MEM(IPD), N, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 2, IAM, PBERR )
*
*           Test PBZSYMM ( CASE 3 )
*
            IBPOS = IBROW
            ICPOS = ICROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Matrix', 'Right', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZSYMM( 'Right', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 3, IAM, PBERR )
*
*           Test PBZSYMM ( CASE 4 )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPB = K * LCA + IPC
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Matrix', 'Right', 'Upper', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                    MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAROW,
     $                    IACOL, IBPOS, ICPOS, ' ', BWORK, CWORK,
     $                    MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = K * N + IPD
            IPA = K * N + IPB
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZSYMM( 'Right', 'Upper', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 4, IAM, PBERR )
*
*           Test PBZSYMM ( CASE 5 )
*
            IBPOS = IBCOL
            ICPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LCC = NUMROC( N, NB, MYCOL, ICPOS, NPCOL )
*
            IPC = 1
            IPB = LRA * LCC + IPC
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPC),
     $                     LRA, IAROW, ICPOS, ICSEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Block', 'Left', 'Lower', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB), MAX(1,K),
     $                    BETA, MEM(IPC), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ICPOS, ACOMM, AWORK, ' ', 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZSYMM( 'Left', 'Lower', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                  MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPC), K, MEM(IPD),
     $                     K, IAROW, ICPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 5, IAM, PBERR )
*
*           Test PBZSYMM ( CASE 6 )
*
            IBPOS = IBROW
            ICPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICPOS, NPROW )
*
            IPC = 1
            IPB = LRC * LCA + IPC
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPC),
     $                     LRC, ICPOS, IACOL, ICSEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYMM( ICONTXT, 'Block', 'Right', 'Upper', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                    MAX(1,LRB), BETA, MEM(IPC), MAX(1,LRC), IAROW,
     $                    IACOL, IBPOS, ICPOS, ACOMM, AWORK, ' ', 0,
     $                    MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPB
            IPB = N * K + IPD
            IPA = N * K + IPB
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZSYMM( 'Right', 'Upper', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                  MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYMM ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZSYMM ', 6, IAM, PBERR )
*
*==================================================================
*           PBZSYRK TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZSYRK ( CASE 1 )
*
            IAPOS = IACOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRC), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYRK( 'Lower', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                  BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 1, IAM, PBERR )
*
*           Test PBZSYRK ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = LRC * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRC), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM,
     $                    AWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYRK( 'Upper', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                  BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 2, IAM, PBERR )
*
*           Test PBZSYRK ( CASE 3 )
*
            IAPOS = IAROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Matrix', 'Lower', 'Trans', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYRK( 'Lower', 'Trans', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 3, IAM, PBERR )
*
*           Test PBZSYRK ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPW = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Matrix', 'Upper', 'Trans', N, K, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,LRC), IAPOS, ICROW, ICCOL, ACOMM, AWORK,
     $                    'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYRK( 'Upper', 'Trans', N, K, ALPHA, MEM(IPA),
     $                  MAX(1,K), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 4, IAM, PBERR )
*
*           Test PBZSYRK ( CASE 5 )
*
            IAPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRC * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,K), BETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL ZSYRK( 'Upper', 'No', K, N, ALPHA, MEM(IPA), MAX(1,K),
     $                  BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 5, IAM, PBERR )
*
*           Test PBZSYRK ( CASE 6 )
*
            IAPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPW = LRA * LCC + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYRK( ICONTXT, 'Block', 'Lower', 'Trans', K, N, NB,
     $                    ALPHA, MEM(IPA), MAX(1,LRA), BETA, MEM(IPC),
     $                    MAX(1,K), IAPOS, ICROW, ICCOL, CCOMM, ' ',
     $                    CWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPW = N * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL ZSYRK( 'Lower',  'Trans', K, N, ALPHA, MEM(IPA),
     $                  MAX(1,N), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYRK ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZSYRK ', 6, IAM, PBERR )
*
*==================================================================
*           PBZSYR2K TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            CCOMM = ACOMM
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
            ABCOMM = ACOMM
*
*           Test PBZSYR2K ( CASE 1 )
*
            IAPOS  = IACOL
            IBPOS  = IBCOL
            LCA = NUMROC( K, KB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Matrix', 'Lower', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), BETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Lower', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 1, IAM, PBERR )
*
*           Test PBZSYR2K ( CASE 2 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRC * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = LRC * K + IPA
            IPW = LRC * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = LRC * K + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Matrix', 'Upper', 'No', N, K, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRC), MEM(IPB),
     $                     MAX(1,LRC), BETA, MEM(IPC), MAX(1,LRC),
     $                     IAPOS, IBPOS, ICROW, ICCOL, ABCOMM, AWORK,
     $                     BWORK, 'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = N * K + IPA
            IPW = N * K + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Upper', 'No', N, K, ALPHA, MEM(IPA), MAX(1,N),
     $                   MEM(IPB), MAX(1,N), BETA, MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 2, IAM, PBERR )
*
*           Test PBZSYR2K ( CASE 3 )
*
            IAPOS = IAROW
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( N, NB, MYROW, ICROW, NPROW )
            LCC = NUMROC( N, NB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Matrix', 'Lower', 'Trans', N,
     $                     K, NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Lower', 'Trans', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 3, IAM, PBERR )
*
*           Test PBZSYR2K ( CASE 4 )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IPB = LRA * LCC + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPB = K * LCC + IPA
            IPW = LRB * LCC + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPW = K * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Matrix', 'Upper', 'Trans', N, K,
     $                     NB, ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,LRC), IAPOS,
     $                     IBPOS, ICROW, ICCOL, ABCOMM, AWORK, BWORK,
     $                     'No', MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * N + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPD),
     $                     N, 0, 0, ICSEED, 0, N, 0, N, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Upper', 'Trans', N, K, ALPHA, MEM(IPA),
     $                   MAX(1,K), MEM(IPB), MAX(1,K), BETA, MEM(IPD),
     $                   MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, N, NB, NB, MEM(IPC), LRC,
     $                     MEM(IPD), N, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 4, IAM, PBERR )
*
*           Test PBZSYR2K ( CASE 5 )
*
            IAPOS = IACOL
            IBPOS = IACOL
            LCA = NUMROC( N, NB, MYCOL, IAPOS, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRC * LCA + IPA
            IPW = LRC * LCB + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     LRC, ICROW, IAPOS, IASEED, 0, LRC, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRC, ICROW, IBPOS, IBSEED, 0, LRC, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Block', 'Upper', 'No', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,K), MEM(IPB),
     $                     MAX(1,K), BETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Upper', 'No', K, N, ALPHA, MEM(IPA), MAX(1,K),
     $                   MEM(IPB), MAX(1,K), BETA, MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 5, IAM, PBERR )
*
*           Test PBZSYR2K ( CASE 6 )
*
            IAPOS = IAROW
            IBPOS = IAROW
            LRA = NUMROC( N, NB, MYROW, IAPOS, NPROW )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
            LRC = NUMROC( K, KB, MYROW, ICROW, NPROW )
            LCC = NUMROC( K, KB, MYCOL, ICCOL, NPCOL )
*
            IPC = 1
            IPA = LRC * LCC + IPC
            IF( LSAME( CWORK, 'Y' ) ) IPA = K * K + IPC
            IPB = LRA * LCC + IPA
            IPW = LRB * LCC + IPB
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     LRA, IAPOS, ICCOL, IASEED, 0, LRA, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, ICCOL, IBSEED, 0, LRB, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPC),
     $                     LRC, ICROW, ICCOL, ICSEED, 0, LRC, 0, LCC,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZSYR2K( ICONTXT, 'Block', 'Lower', 'Trans', K, N, NB,
     $                     ALPHA, MEM(IPA), MAX(1,LRA), MEM(IPB),
     $                     MAX(1,LRB), BETA, MEM(IPC), MAX(1,K), IAPOS,
     $                     IBPOS, ICROW, ICCOL, CCOMM, ' ', ' ', CWORK,
     $                     0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * K + IPD
            IPB = K * N + IPA
            IPW = K * N + IPB
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPA),
     $                     N, 0, 0, IASEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPD),
     $                     K, 0, 0, ICSEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL ZSYR2K( 'Lower',  'Trans', K, N, ALPHA, MEM(IPA),
     $                   MAX(1,N), MEM(IPB), MAX(1,N), BETA, MEM(IPD),
     $                   MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, K, KB, KB, MEM(IPC), K, MEM(IPD),
     $                     K, ICROW, ICCOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZSYR2K ', 6, IAM, PBERR )
            CALL PBXERPNT( NOUT, 'PBZSYR2K ', 6, IAM, PBERR )
*
*==================================================================
*           PBZTRMM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZTRMM ( CASE 1 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Lower', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 1, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 2 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 2, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 3 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Lower', 'Trans', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 3, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 4 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Upper', 'Trans', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 4, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 5 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Lower', 'No', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 5, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 6 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Upper', 'No', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 6, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 7 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 7, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 8 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Upper', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 8, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 9 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Block', 'Left', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 9, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 10 )
*
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Block', 'Right', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,LRB), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRB,
     $                     MEM(IPD), N, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 10, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 11 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Conju',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N,  0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Lower', 'Conju', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 11, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 12 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Conju',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Upper', 'Conju', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 12, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 13 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Lower', 'Conju', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 13, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 14 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), LRA, IAROW, IACOL, IASEED, 0, LRA,
     $                     0, LCA, MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    BCOMM, BWORK, MULLEN, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = K * N + IPD
            IPW = N * N + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB,
     $                     MEM(IPA), N, 0, 0, IASEED, 0, N, 0, N, 0, 0,
     $                     1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Right', 'Upper', 'Conju', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,N), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 14, IAM, PBERR )
*
*           Test PBZTRMM ( CASE 15 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRMM( ICONTXT, 'Block', 'Left', 'Lower', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, 0, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRMM( 'Left', 'Lower', 'Conju', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRMM ', 15, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZTRMM ', 15, IAM, PBERR )
*
*==================================================================
*           PBZTRSM TEST
*==================================================================
*
            IF( PBERR ) THEN
               PBALL = .TRUE.
               PBERR = .FALSE.
            END IF
            TOL = 100.0D0 * DBLE(N) * DLAMCH('eps')
*
*           Test PBZTRSM ( CASE 1 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'No',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 1, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 2 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'No',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 2, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 3 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Trans',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 3, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 4 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Trans',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 4, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 5 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'No',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 5, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 6 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'No',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, ' ', 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 6, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 7 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 7, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 8 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Trans',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 8, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 9 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Block', 'Left', 'Lower', 'Trans',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRSM( 'Left', 'Lower', 'Trans', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 9, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 10 )
*
            IBPOS = IBROW
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( N, NB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Block', 'Right', 'Upper', 'No',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,LRB), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL ZTRSM( 'Right', 'Upper', 'No', DIAG, N, K, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,N) )
*
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRB,
     $                     MEM(IPD), N, IBPOS, IACOL, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 10, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 11 )
*
            IBPOS = IBCOL
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( K, KB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Conju',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Lower', 'Conju',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N, 0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 11, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 12 )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = LRA * K + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Conju',
     $                    DIAG, N, K, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,LRA), IAROW, IACOL, IBPOS,
     $                    ' ', BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRA*LCB.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRA*LCB, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Left', 'Upper', 'Conju',
     $                    DIAG, N, K, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,LRA), IAROW,
     $                    IACOL, IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
*
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, K, NB, KB, MEM(IPD),
     $                     N,  0, 0, IBSEED, 0, N, 0, K, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, N, K, NB, KB, MEM(IPB), LRA,
     $                     MEM(IPD), N, IAROW, IBPOS, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 12, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 13 )
*
            IBPOS = IBROW
            LRA = NUMROC( N, NB, MYROW, IAROW, NPROW )
            LCA = NUMROC( N, NB, MYCOL, IACOL, NPCOL )
            LRB = NUMROC( K, KB, MYROW, IBPOS, NPROW )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Lower', 'Conju',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 13, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 14 )
*
            IPB = 1
            IPA = LRB * LCA + IPB
            IF( LSAME( BWORK, 'Y' ) ) IPA = K * LCA + IPB
            IPW = LRA * LCA + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', N, N, NB, NB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRB, IBPOS, IACOL, IBSEED, 0, LRB, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,LRA),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS, ' ',
     $                    BWORK, MEM(IPW) )
*
*           Find the maximum value of MEM(IPB)
*
            IF( LRB*LCA.GT.0 ) THEN
              XMAX = ABS( MEM( IPB-1+IZAMAX( LRB*LCA, MEM(IPB), 1 ) ) )
            ELSE
              XMAX = TZERO
            END IF
            CALL DGAMX2D( ICONTXT, 'All', 'H-cube', 1, 1, XMAX, 1, IX,
     $                    JX, 1, -1, -1 )
            TOLL = XMAX * TOL
*
*           Do reverse computation with PBZTRMM
*
            CALL PBZTRMM( ICONTXT, 'Matrix', 'Right', 'Upper', 'Conju',
     $                    DIAG, K, N, NB, ONE/ALPHA, MEM(IPA),
     $                    MAX(1,LRA), MEM(IPB), MAX(1,K), IAROW, IACOL,
     $                    IBPOS, BCOMM, 'No', MULLEN, MEM(IPW) )
            IPD = IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IBPOS, IACOL, TOLL, ERROR )
            IF( ERROR.GT.TOLL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 14, IAM, PBERR )
*
*           Test PBZTRSM ( CASE 15 )
*
            IBPOS = IBCOL
            LRA = NUMROC( K, KB, MYROW, IAROW, NPROW )
            LCA = NUMROC( K, KB, MYCOL, IACOL, NPCOL )
            LCB = NUMROC( N, NB, MYCOL, IBPOS, NPCOL )
*
            IPB = 1
            IPA = LRA * LCB + IPB
            IPW = LRA * LCA + IPA
            IF( LSAME( AWORK, 'Y' ) ) IPW = K * K + IPA
*
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     LRA, IAROW, IACOL, IASEED, 0, LRA, 0, LCA,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPB),
     $                     LRA, IAROW, IBPOS, IBSEED, 0, LRA, 0, LCB,
     $                     MYROW, MYCOL, NPROW, NPCOL )
            CALL PBZTRSM( ICONTXT, 'Block', 'Left', 'Lower', 'Conju',
     $                    DIAG, K, N, NB, ALPHA, MEM(IPA), MAX(1,K),
     $                    MEM(IPB), MAX(1,K), IAROW, IACOL, IBPOS,
     $                    ACOMM, AWORK, MEM(IPW) )
*
*           Generate data for sequential computation
*
            IPD = IPA
            IPA = N * K + IPD
            IPW = K * K + IPA
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, K, KB, KB, MEM(IPA),
     $                     K, 0, 0, IASEED, 0, K, 0, K, 0, 0, 1, 1 )
            CALL PZMATGEN( ICONTXT, 'No', 'No', K, N, KB, NB, MEM(IPD),
     $                     K, 0, 0, IBSEED, 0, K, 0, N, 0, 0, 1, 1 )
            CALL ZTRSM( 'Left', 'Lower', 'Conju', DIAG, K, N, ALPHA,
     $                  MEM(IPA), MAX(1,K), MEM(IPD), MAX(1,K) )
*
            CALL PZMATCHK( ICONTXT, K, N, KB, NB, MEM(IPB), K, MEM(IPD),
     $                     K, IAROW, IBPOS, TOL, ERROR )
            IF( ERROR.GT.TOL )
     $        CALL PBXERBLA( NOUT, 'PBZTRSM ', 15, IAM, PBERR )
*
            CALL PBXERPNT( NOUT, 'PBZTRSM ', 15, IAM, PBERR )
*
            INTEST = INTEST + 1
            IF( IAM.EQ.0 ) WRITE( NOUT, 4000 ) INTEST
   10     CONTINUE
   20   CONTINUE
        CALL BLACS_GRIDEXIT( ICONTXT )
   30 CONTINUE
*
      IF( IAM.EQ.0 ) THEN
         IF( PBALL ) THEN
            WRITE( NOUT, 5000 )
         ELSE
            WRITE( NOUT, 6000 ) INTEST
         END IF
         WRITE( NOUT, 7000 )
      END IF
*
      CALL BLACS_EXIT( 0 )
*
 1000 FORMAT( /, 'Illegal input value of ', A11 )
 2000 FORMAT(/, '    P    Q     M    N    K    MB  NB  KB')
 3000 FORMAT(1X,I4,1X,I4,3X,I4,1X,I4,1X,I4,2X,I3,1X,I3,1X,I3)
 4000 FORMAT( '-- PBZ3CHECK TEST SET (', I4, ' ) IS COMPLETED --')
 5000 FORMAT( /, '== THERE WERE FAILURES IN THIS RUN ==')
 6000 FORMAT( /, '== THERE WERE NO FAILURES IN (', I4, ' ) ',
     $        'TEST SETS ==')
 7000 FORMAT( /, 'End of PBZ3CHECK TESTS.' )
*
      STOP
*
*     End of PBZ3CHECK
*
      END
